package bloques;

import java.util.*;
import javax.swing.undo.*;
import interfaz.deshacer.*;
import java.io.Serializable;

/**
 * <p>Title: Bloque.java</p>
 * <p>Description: Clase abastracta de los bloques</p>
 * @author Diego Daz Garca
 * @version 1.0
 */

public abstract class Bloque
    extends java.util.Observable
    implements Serializable {
  // serialVersionUID version 1.3 (y actual): -1021967636037645479L
  // serialVersionUID version 1.4: 9122061219071415581L
  static final long serialVersionUID = -1021967636037645479L;
  public ArrayList componentes = new ArrayList();
  private String nombre = " Bloque clase padre ";
  private Bloque padre;

  /**Constructor de la clase abstracta bloque
   * @param nombreinicial Es el nombre del nuevo bloque
   * @param padre_ Bloque padre del bloque a crear*/
  public Bloque(String nombreinicial, Bloque padre_) {
    nombre = nombreinicial;
    padre = padre_;
    asignarNombreAutomaticamente();
  }

  /**Cambia el nombre de un diagrama sin comprobar que no haya otro
   * bloque de igual nombre
   * @param nuevo nombre nuevo del bloque*/
  public void cambiarNombreSinComprobacion(String nuevo) {
    nombre = nuevo;
  }

  /**Calcula el mttf del bloque
   * @param reparable Indica la reparabilidad o no del bloque
   * @return El Mttf del bloque*/
  public abstract double obtenerMttf(boolean reparable);

  /**Calcula la tasa de fallos del bloque
   * @param reparable Indica la reparabilidad o no del bloque
   * @return La tasa de fallos del bloque*/
  public abstract double obtenerLambda(boolean reparable);

  /**Calcula el mtbf del bloque
   * @param reparable Indica la reparabilidad o no del bloque
   * @return El Mtbf del bloque*/
  public abstract double obtenerMtbf(boolean reparable);

  /**Calcula la tasa de reparacin del bloque
   * @return La tasa de reparacin del bloque*/
  public abstract double obtenerMu();

  /**Calcula el mttr del bloque
   * @return El Mttr del bloque*/
  public abstract double obtenerMttr();

  /**Calcula la fiabilidad del bloque
   * @param t Valor del tiempo para el clculo de la fiabilidad
   * @param reparable Indica la reparabilidad o no del bloque
   * @return La fiabilidad del bloque*/
  public abstract double obtenerFiabilidad(double t, boolean reparable);

  /**Calcula la disponibilidad del bloque
   * @return La disponibilidad del bloque*/
  public abstract double obtenerDisponibilidad();

  /** Calcula la fiabilidad equivalente, es decir, la fiabilidad considerando
   * que el bloque sea exponencial, lo sea o no. Para ello utiliza lambda
  * @param t Valor del tiempo para el clculo de la fiabilidad
  * @param reparable Indica la reparabilidad o no del bloque
  * @return La fiabilidad equivalente del bloque */
  public double obtenerFiabilidadEq(double t, boolean reparable) {
    return Math.exp( -obtenerLambda(reparable) * t);
  }

  /**Calcula la no disponibilidad del bloque
   * @return La no disponibilidad del bloque*/
  public double obtenerNoDisponibilidad() {
    return 1 - obtenerDisponibilidad();
  };

  /**Comprueba la validez para el clculo de un bloque
   * @param reparable Cierto si el bloque es reparable, falso en caso contrario
   * @return Un string que estar vaci si el diagrama est correcto para
   * realizar clculos y que en caso contrario contendr el motivo por el
   * cul no est correcto*/
  public abstract String comprobar(boolean reparable);

  /**Crea un bloque simple hijo del bloque actual
   * @return El bloque creado*/
  public Bloque bloqueSimple() {
    return null;
  }

  /**Crea un bloque expansible hijo del bloque actual
   * @param paralelo Cierto si el nuevo bloque es en paralelo, falso si es en
   * serie
   * @return El bloque creado*/
  public Bloque bloqueExpansible(boolean paralelo) {
    return null;
  }

  /**Elimina un bloque
   * @param posicion Posicin del bloque a eliminar dentro de la lista de hijos
   * del bloque padre
   * @return Cierto si se ha eliminado el bloque, falso en caso contrario*/
  public boolean eliminaBloque(int posicion) {
    return false;
  }

  /**Devuelve si un bloque es extensible o no
   * @return True si es extensible, false si no*/
  public boolean esExtensible() {
    return false;
  }

  /**Devuelve si un bloque es paralelo o no
   * @return True si es paralelo, false si no*/
  public boolean esParalelo() {
    return false;
  }

  /**Devuelve el nombre del bloque
   * @return Un string con el nombre del bloque*/
  public String nombre() {
    return nombre;
  }

  /**Aade un bloque a la lista de hijos
   * @param b Bloque a aadir
   * @param posicion Posicion que ocupa el bloque dentro de la lista de hijos*/
  public void add(Bloque b, int posicion) {
    return;
  }

  /**Produce si el bloque tiene padre o no(bloque principal)
   * @return Cierto si tiene padre, falso si es el bloque principal*/
  public boolean tienePadre() {
    return (padre != null);
  }

  /**Devuelve el padre del diagrama
   * @return El bloque padre del diagrama*/
  public Bloque padre() {
    return padre;
  }

  /**Hace que un bloque pase a tener otro padre
   * @param nuevoPadre Nuevo padre del bloque*/
  public void asignarPadre(Bloque nuevoPadre) {
    padre = nuevoPadre;
  }

  /**Cambia el nombre del bloque tras comprobar que no hay otro con ese nombre
   * @param nuevoNombre Nuevo nombre del bloque
   * @return Cierto si se ha producido el cambio, falso si el nombre no era
   * vlido*/
  public boolean cambiarNombre(String nombreNuevo) {
    boolean valido = false;
    if ((nombreNuevo == nombre) || nombreValido(nombreNuevo)) {
      String nombreViejo = nombre;
      nombre = nombreNuevo;
      String[] nombres = new String[2];
      nombres[0] = nombreViejo;
      nombres[1] = nombreNuevo;
      setChanged();
      notifyObservers(nombres);
      valido = true;
    }
    return valido;
  }

  /**Comprueba que no haya ningn bloque con el mismo nombre que el que se le
   * pasa como parmetro
   * @param nombre Nombre a comprobar
   * @return Cierto si el nombre es vlido (no hay un bloque que lo use), falso
   * en caso contrario*/
  public boolean nombreValido(String nombre) {
    boolean nombreYaExistente = false;
    // obtener bloque padre principal
    Bloque bloquePadrePrincipal = this;
    while (bloquePadrePrincipal.padre != null) {
      bloquePadrePrincipal = bloquePadrePrincipal.padre;

    }
    nombreYaExistente = encontrarNombre(bloquePadrePrincipal, nombre);
    return!nombreYaExistente;
  }

  /**Funcin que dado un bloque y un nombre nos dice si el bloque o alguno
   * de sus descendientes tiene ese nombre
   * @param bloqueP Bloque que vamos a evaluar
   * @param nombreBuscado Nombre que vamos a buscar
   * @return Cierto si el nombre es el nombre del bloqueP o de alguno de sus
   * hijos, falso en caso contrario */
  public boolean encontrarNombre(Bloque bloqueP, String nombreBuscado) {
    boolean encontrado = false;
    if (bloqueP.nombre().compareTo(nombreBuscado) == 0) {
      encontrado = true;

    }
    else if (bloqueP.esExtensible()) {
      int i;
      for (i = 0; i < bloqueP.componentes.size(); i++) {
        encontrado = encontrado ||
            encontrarNombre( (Bloque) bloqueP.componentes.get(i), nombreBuscado);
        if (encontrado == true) {
          break;
        }
      }
    }
    return encontrado;
  }

  /**Asigna a los bloques el nombre "Bloque i", comprobando que ese nombre aun
   * no est asignado a otro bloque e incrementando la i en una unidad en
   * caso de que lo est*/
  private void asignarNombreAutomaticamente() {
    int i = 1;
    String nuevoNombre = "Bloque " + i;

    while (!nombreValido(nuevoNombre)) {
      i++;
      nuevoNombre = "Bloque " + i;
    }

    cambiarNombre(nuevoNombre);
  }

  public String toString() {
    return nombre;
  }
}
