package interfaz;

import interfaz.deshacer.ElementoDefinirBloqueEdit;

import java.awt.Color;
import java.awt.Frame;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.text.DecimalFormat;
import java.text.NumberFormat;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFormattedTextField;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.SwingConstants;
import javax.swing.border.TitledBorder;
import javax.swing.text.NumberFormatter;
import javax.swing.undo.UndoableEdit;

import util.DatosBloque;
import bloques.BloqueBasico;

/**
 * <p>Title: DefinirBloque.java</p>
 * <p>Description: Define los dilogos en los que se modifican los parmetros
 * que definen los bloques simples</p>
 * @author Diego Daz Garca
 * @version 1.0
 */

public class DefinirBloque
    extends JDialog
    implements ActionListener,
    PropertyChangeListener {

  // Valores para los campos de texto
  double MTTF = 0;
  double Lambda = 0;
  double MTBF = 0;
  double Mu = 0;
  double MTTR = 0;

  // Campos para la entrada de datos
  JFormattedTextField campoTasaFallos;
  JFormattedTextField campoMTTF;
  JFormattedTextField campoMTBF;
  JFormattedTextField campoMTTR;
  JFormattedTextField campoTasaReparacion;

  // Formato para los valores de los campos
  private NumberFormat formatoValores;

  JPanel jPanel1 = new JPanel();
  JRadioButton botonTasaFallos = new JRadioButton();
  JRadioButton botonMTTF = new JRadioButton();
  JRadioButton botonMTBF = new JRadioButton();
  TitledBorder titledBorder1;
  TitledBorder titledBorder2;
  JPanel jPanel2 = new JPanel();
  JRadioButton botonTasaReparacion = new JRadioButton();
  JRadioButton botonMTTR = new JRadioButton();
  JButton botonCancelar = new JButton();
  JButton botonAceptar = new JButton();
  ButtonGroup botonesComunes = new ButtonGroup();
  ButtonGroup botonesSistemasReparables = new ButtonGroup();
  BloqueBasico bloqueaDefinir;
  // Guardamos los datos del bloque antes de modificarlos
  DatosBloque datosAnterioresBloque;
  // Al salir aqu estarn los datos del bloque ya modificado
  boolean reparable = false;

  /**Constructor
   * @param frame Ventana Principal de la aplicacin
   * @param bloqueaDefinir_ Bloque cuyos parmetros representa*/
  public DefinirBloque(Frame frame, BloqueBasico bloqueaDefinir_) {
    super(frame, "Parmetros del bloque " + bloqueaDefinir_.nombre(), false);
    bloqueaDefinir = bloqueaDefinir_;
    datosAnterioresBloque = new DatosBloque(bloqueaDefinir, reparable);
    try {
      jbInit();
      pack();
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  /**Inicializa las instancias de la clase
   * @throws Exception */
  private void jbInit() throws Exception {
    // Inicializo los valores de los campos
    MTTF = bloqueaDefinir.obtenerMttf(reparable);
    Lambda = bloqueaDefinir.obtenerLambda(reparable);
    Mu = bloqueaDefinir.obtenerMu();
    MTTR = bloqueaDefinir.obtenerMttr();
    MTBF = bloqueaDefinir.obtenerMtbf(reparable);

    // se prepara el formato que van a tener los campos
    formatoValores = new DecimalFormat("#0.0############");
    NumberFormatter mascaraDouble = new NumberFormatter(formatoValores);
    mascaraDouble.setOverwriteMode(false);
    mascaraDouble.setAllowsInvalid(true);

    campoTasaFallos = new JFormattedTextField(mascaraDouble);
    campoTasaFallos.setValue(new Double(Lambda));
    campoTasaFallos.addPropertyChangeListener("value", this);

    campoMTTF = new JFormattedTextField(mascaraDouble);
    campoMTTF.setValue(new Double(MTTF));
    campoMTTF.setEnabled(true);
    campoMTTF.addPropertyChangeListener("value", this);

    campoMTBF = new JFormattedTextField(mascaraDouble);
    campoMTBF.setValue(new Double(MTBF));
    campoMTBF.addPropertyChangeListener("value", this);

    campoMTTR = new JFormattedTextField(mascaraDouble);
    campoMTTR.setValue(new Double(MTTR));
    campoMTTR.setEnabled(true);
    campoMTTR.addPropertyChangeListener("value", this);

    campoTasaReparacion = new JFormattedTextField(mascaraDouble);
    campoTasaReparacion.setValue(new Double(Mu));
    campoTasaReparacion.addPropertyChangeListener("value", this);

    campoTasaFallos.setEnabled(false);
    campoMTBF.setEnabled(false);
    campoTasaReparacion.setEnabled(false);

    campoMTTF.setScrollOffset(0);
    botonMTTF.setSelected(true);
    botonTasaFallos.setBorderPainted(false);
    botonTasaFallos.setContentAreaFilled(true);
    this.setEnabled(true);
    botonTasaReparacion.setSelected(false);
    botonMTTR.setSelected(true);
    botonesComunes.add(botonTasaFallos);
    botonesComunes.add(botonMTTF);
    botonesComunes.add(botonMTBF);
    botonesSistemasReparables.add(botonTasaReparacion);
    botonesSistemasReparables.add(botonMTTR);
    titledBorder1 = new TitledBorder(BorderFactory.createEtchedBorder(Color.
        white, new Color(165, 163, 151)), "Fallos");
    titledBorder2 = new TitledBorder(BorderFactory.createEtchedBorder(Color.
        white, new Color(165, 163, 151)), "Reparacin");
    this.getContentPane().setLayout(null);
    jPanel1.setFont(new java.awt.Font("Dialog", 0, 12));
    jPanel1.setBorder(titledBorder1);
    jPanel1.setBounds(new Rectangle(14, 9, 309, 123));
    jPanel1.setLayout(null);
    botonTasaFallos.setFont(new java.awt.Font("Dialog", 0, 12));
    botonTasaFallos.setText(" Tasa de fallos");
    botonTasaFallos.setBounds(new Rectangle(12, 52, 141, 23));
    campoTasaFallos.setFont(new java.awt.Font("Dialog", 0, 12));
    campoTasaFallos.setHorizontalAlignment(SwingConstants.RIGHT);
    campoTasaFallos.setBounds(new Rectangle(171, 54, 122, 21));
    botonMTTF.setBounds(new Rectangle(12, 20, 141, 23));
    botonMTTF.setText(" MTTF");
    botonMTTF.setFont(new java.awt.Font("Dialog", 0, 12));
    campoMTTF.setBounds(new Rectangle(171, 22, 122, 21));
    campoMTTF.setHorizontalAlignment(SwingConstants.RIGHT);
    campoMTTF.setFont(new java.awt.Font("Dialog", 0, 12));
    botonMTBF.setBounds(new Rectangle(12, 84, 141, 23));
    botonMTBF.setText(" MTBF");
    botonMTBF.setFont(new java.awt.Font("Dialog", 0, 12));
    campoMTBF.setBounds(new Rectangle(171, 86, 122, 21));
    campoMTBF.setHorizontalAlignment(SwingConstants.RIGHT);
    campoMTBF.setFont(new java.awt.Font("Dialog", 0, 12));
    jPanel2.setLayout(null);
    jPanel2.setBounds(new Rectangle(16, 142, 309, 96));
    jPanel2.setBorder(titledBorder2);
    jPanel2.setFont(new java.awt.Font("Dialog", 0, 12));
    campoMTTR.setBounds(new Rectangle(170, 25, 123, 21));
    campoMTTR.setHorizontalAlignment(SwingConstants.RIGHT);
    campoMTTR.setFont(new java.awt.Font("Dialog", 0, 12));
    botonTasaReparacion.setFont(new java.awt.Font("Dialog", 0, 12));
    botonTasaReparacion.setText(" Tasa de reparacin");
    botonTasaReparacion.setBounds(new Rectangle(12, 58, 158, 23));
    campoTasaReparacion.setFont(new java.awt.Font("Dialog", 0, 12));
    campoTasaReparacion.setHorizontalAlignment(SwingConstants.RIGHT);
    campoTasaReparacion.setBounds(new Rectangle(171, 60, 122, 21));
    botonMTTR.setBounds(new Rectangle(12, 23, 141, 23));
    botonMTTR.setText(" MTTR");
    botonMTTR.setFont(new java.awt.Font("Dialog", 0, 12));
    botonCancelar.setBounds(new Rectangle(229, 246, 95, 32));
    botonCancelar.setText("Cancelar");
    botonAceptar.setText("Aceptar");
    botonAceptar.setBounds(new Rectangle(127, 246, 95, 32));
    this.setResizable(false);
    jPanel1.add(botonMTBF, null);
    jPanel1.add(campoMTBF, null);
    jPanel1.add(campoTasaFallos, null);
    jPanel1.add(botonTasaFallos, null);
    jPanel1.add(campoMTTF, null);
    jPanel1.add(botonMTTF, null);
    this.getContentPane().add(jPanel2, null);
    jPanel2.add(botonTasaReparacion, null);
    jPanel2.add(campoTasaReparacion, null);
    jPanel2.add(botonMTTR, null);
    jPanel2.add(campoMTTR, null);
    this.getContentPane().add(botonCancelar, null);
    this.getContentPane().add(botonAceptar, null);
    this.getContentPane().add(jPanel1, null);
    botonAceptar.addActionListener(this);
    botonCancelar.addActionListener(this);
    botonTasaFallos.addActionListener(this);
    botonMTTF.addActionListener(this);
    botonMTBF.addActionListener(this);
    botonTasaReparacion.addActionListener(this);
    botonMTTR.addActionListener(this);
    getRootPane().setDefaultButton(botonAceptar);
    botonAceptar.setDefaultCapable(true);
    pack();
    campoMTTF.requestFocusInWindow();
  }

  static boolean estoyCambiando = false;

  /** Llamada cuando cambian los valores de algn campo
   * @param e Cambio producido en los campos*/
  public void propertyChange(PropertyChangeEvent e) {
    if (estoyCambiando) { // Para evitar que se llame dos veces
      return;
    }
    else {
      estoyCambiando = true;

    }
    Object source = e.getSource();
    if (source == campoTasaFallos) {
      Lambda = ( (Number) campoTasaFallos.getValue()).doubleValue();
      bloqueaDefinir.asignarLambda(Lambda);
      campoMTTF.setValue
          (new Double(bloqueaDefinir.obtenerMttf(reparable)));
      campoMTBF.setValue(new Double(bloqueaDefinir.obtenerMtbf(reparable)));

    }
    else if (source == campoMTTF) {
      MTTF = ( (Number) campoMTTF.getValue()).doubleValue();
      bloqueaDefinir.asignarMttf(MTTF);
      campoTasaFallos.setValue
          (new Double(bloqueaDefinir.obtenerLambda(reparable)));
      campoMTBF.setValue(
          new Double(bloqueaDefinir.obtenerMtbf(reparable)));
    }
    else if (source == campoMTBF) {
      MTBF = ( (Number) campoMTBF.getValue()).doubleValue();
      bloqueaDefinir.asignarMtbf(MTBF);
      if (bloqueaDefinir.obtenerLambda(reparable) != 0 ||
           bloqueaDefinir.obtenerMttr() != 0)
        bloqueaDefinir.asignarMttr(MTBF - MTTF);
      campoMTTR.setValue
          (new Double(bloqueaDefinir.obtenerMttr()));
    }

    if (source == campoTasaReparacion) {
      Mu = ( (Number) campoTasaReparacion.getValue()).doubleValue();
      bloqueaDefinir.asignarMu(Mu);
      campoMTTR.setValue(new Double(bloqueaDefinir.obtenerMttr()));
      campoMTBF.setValue(new Double(bloqueaDefinir.obtenerMtbf(reparable)));
    }
    else if (source == campoMTTR) {
      MTTR = ( (Number) campoMTTR.getValue()).doubleValue();
      bloqueaDefinir.asignarMttr(MTTR);
      campoTasaReparacion.setValue
          (new Double(bloqueaDefinir.obtenerMu()));
      campoMTBF.setValue(new Double(bloqueaDefinir.obtenerMtbf(reparable)));
    }
    estoyCambiando = false;
  }

  /**Permite procesar el evento de cerrar ventana
   * @param e Evento de cerrar ventana*/
  protected void processWindowEvent(WindowEvent e) {
    if (e.getID() == WindowEvent.WINDOW_CLOSING) {
      cancelar();
    }
    super.processWindowEvent(e);
  }

  //Cierra el dialogo
  void cancel() {
    dispose();
  }

  /** Funcin llamada a travs del botn aceptar, se encarga de procesar el
   * ltimo cambio realizado en los campos
   * @throws Exception */
  private void procesarUltimoCambio() throws Exception {
    if (botonTasaFallos.isSelected()) {
      campoTasaFallos.commitEdit();
    }
    if (botonMTTF.isSelected()) {
      campoMTTF.commitEdit();
    }
    if (botonMTBF.isSelected()) {
      campoMTBF.commitEdit();
    }
    if (botonTasaReparacion.isSelected()) {
      campoTasaReparacion.commitEdit();
    }
    if (botonMTTR.isSelected()) {
      campoMTTR.commitEdit();
    }
  }

  public void actionPerformed(ActionEvent e) {
    if (e.getSource() == botonAceptar) {
      try {
        procesarUltimoCambio();
      }
      catch (Exception exception) {
      }
      if ( (MTBF < 0) || (Lambda < 0) || (MTTR < 0) || (MTTF < 0) || (Mu < 0)) {
        // algn valor incorrecto
        JOptionPane.showMessageDialog(null,
                                      "Lambda, Mu, MTTF, MTBF y MTTR " +
                                      "no pueden tener valores negativos",
                                      "Valores incorrectos",
                                      JOptionPane.ERROR_MESSAGE);
      }
      else { // todos los valores correctos
        DatosBloque datosFinalesBloque = new DatosBloque(bloqueaDefinir,
            reparable);
        UndoableEdit edit = new ElementoDefinirBloqueEdit(bloqueaDefinir,
            datosAnterioresBloque, datosFinalesBloque);
        // Se coloca el evento en el soporte de operaciones de deshacer/rehacer
        interfaz.FramePrincipal.undoSupport_.postEdit(edit);
        cancel();
      }
    }
    if (e.getSource() == botonCancelar) {
      cancelar();
    }
    if (e.getSource() == botonTasaFallos) {
      campoTasaFallos.setEnabled(true);
      campoMTTF.setEnabled(false);
      campoMTBF.setEnabled(false);
    }
    if (e.getSource() == botonMTTF) {
      campoTasaFallos.setEnabled(false);
      campoMTTF.setEnabled(true);
      campoMTBF.setEnabled(false);
    }
    if (e.getSource() == botonMTBF) {
      campoTasaFallos.setEnabled(false);
      campoMTTF.setEnabled(false);
      campoMTBF.setEnabled(true);
    }
    if (e.getSource() == botonTasaReparacion) {
      campoTasaReparacion.setEnabled(true);
      campoMTTR.setEnabled(false);
    }
    if (e.getSource() == botonMTTR) {
      campoTasaReparacion.setEnabled(false);
      campoMTTR.setEnabled(true);
    }
  }

  /**Reestablece los valores iniciales cuando decidimos apretar el botn de
   * cancelar*/
  private void cancelar() {
    if (datosAnterioresBloque.definidoLambda()) {
      bloqueaDefinir.asignarLambda(datosAnterioresBloque.metricaFallos());
    }
    else {
      bloqueaDefinir.asignarMttf(datosAnterioresBloque.metricaFallos());
    }
    if (datosAnterioresBloque.definidoMu()) {
      bloqueaDefinir.asignarMu(datosAnterioresBloque.metricaReparabilidad());
    }
    else {
      bloqueaDefinir.asignarMttr(datosAnterioresBloque.metricaReparabilidad());
    }
    bloqueaDefinir.asignarMtbf(datosAnterioresBloque.valorMtbf());
    cancel();
  }
}
