package interfaz;

import interfaz.deshacer.EditarCortarEdit;
import interfaz.deshacer.EditarPegarEdit;
import interfaz.deshacer.ElementoBloqueExpansibleEdit;
import interfaz.deshacer.ElementoBloqueSimpleEdit;
import interfaz.deshacer.ElementoEliminarEdit;

import java.awt.BasicStroke;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Observable;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.KeyStroke;
import javax.swing.undo.UndoableEdit;

import bloques.Bloque;
import bloques.BloqueBasico;
import bloques.BloqueParalelo;
import bloques.BloqueSerie;
import java.awt.Color;

/**
 * <p>Title: Diagrama.java</p>
 * <p>Description: Clase que define el lienzo en el cul se dibujarn
 * los diagramas</p>
 * @author Diego Daz Garca
 * @version 1.0
 */

public class Diagrama
    extends JPanel
    implements java.util.Observer {
  public static final int IMPORTADO = 1;
  public static final int PEGADO = 0;
  Point puntoInicio = new Point(0, 0); // posicion donde empezar a dibujar
  private int anchoBloque; // ancho de un bloque
  private int altoBloque; // alto de un bloque
  private int anchoTerminal; // ancho y alto de los terminales
  // espacio entre bloques de la misma columna (en paralelo)
  private int entreBloquesY;
  // espacio entre bloques de la misma fila (en serie)
  private int entreBloquesX;
  private int entreFilas; // espacio entre filas de bloques (serie)
  // espacio entre los dos bloques que forman la figura de bloque extensible
  private int espacioExtensibles;
  // Espacio libre entre el dibujo y el fin del Pane, para evitar que parte del
  // diagrama quede tapada por las barras de scroll
  private int margenScroll;
  private int numBloques; // nmero de bloques
  private boolean paralelo = true; // disposicin de los bloques
  Point primerTerminal = new Point(0, 0); // posicin del primer terminal
  Point ultimoTerminal = new Point(0, 0); // posicin del ltimo terminal
  public Point puntoLimite; // punto ms extremo del diagrama
  ArrayList listaBloques = new ArrayList();
  // El bloque cuyo contenido vamos a dibujar
  private Bloque modelo;
  public ArrayList modelosFiguraBloque = new ArrayList();
  private JPopupMenu popFiguraBloque = new JPopupMenu();
  private JPopupMenu popNoFiguraBloque = new JPopupMenu();
  EscuchadorMenusPopup mpl = new EscuchadorMenusPopup();
  public static FiguraBloque figuraSeleccionada;
  private Diagrama diagramaVisible = this;
  private FramePrincipal ventanaPrincipal;
  private Bloque cortarCopiar = null;
  private boolean puedePegar;
  // Items de los jMenusPopUp que han de poder ser actualizados
  private JMenuItem definirBloque = new JMenuItem();
  private JMenuItem pegar = new JMenuItem();
  private JMenuItem pegarNoFigura = new JMenuItem();
  private JMenuItem pegadoEspecial = new JMenuItem();
  private JMenuItem pegadoEspecialNoFigura = new JMenuItem();
  File archivoCopiarCortar = new File("cortarCopiar.sav");
  boolean reparable = false;

  public void update(Observable t, Object o) {
    listaBloques = modelo.componentes;
    numBloques = modelo.componentes.size();
    ventanaPrincipal.modificadoSinGuardar = true;
    repaint();
  }

  public Diagrama(Bloque bloqueVisible_, FramePrincipal ventanaPrincipal_) {
    super(new BorderLayout());
    modelo = bloqueVisible_;
    ventanaPrincipal = ventanaPrincipal_;
    try {
      jbInit();
    }
    catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  void jbInit() throws Exception {
    prepararMenusPopup();
    addMouseListener(mpl);

    modelo.addObserver(this);

    /*Es necesariio actualizar los valores de las listas para los casos en los
     que el diagrama se cree a partir de un diagrama creado anteriormente y
     guardado en disco*/
    listaBloques.clear();
    modelosFiguraBloque.clear();

    listaBloques = modelo.componentes;
    Iterator iterador = listaBloques.iterator();
    while (iterador.hasNext()) {
      FiguraBloque nuevaFigura = new FiguraBloque(modelo,
                                                  (Bloque) iterador.next(), this);
      modelosFiguraBloque.add(nuevaFigura);
    }
    numBloques = listaBloques.size();
    incorporarAccionesTeclado();
    repaint();
  }

  public void paint(Graphics g) {
    super.paint(g);
    // Pone como color de fondo el elegido en preferencias
    this.setBackground(interfaz.FramePrincipal.prefVars.obtenerColorFondo());
    // Inicializa los tamaos de bloques y lineas
    elegirTamanioBloques();
    elegirAnchoLineas(g);

    // Pone como color de las lineas el elegido en preferencias
    g.setColor(interfaz.FramePrincipal.prefVars.obtenerColorLineas());

    if (modelo.esParalelo() == false) {
      paralelo = false;
    }
    else {
      paralelo = true;

    }
    int numFilas = 0, bloquesPorFila = 0;
    Dimension tamanio = Tamanio();

    // sito correctamente los puntos de inicio para que siempre se dibujen centrados los bloques
    // sito tambin los terminales

    if (paralelo) { // Para bloques en paralelo
      situarPuntosInicioParalelo(puntoInicio, primerTerminal, ultimoTerminal);
    }
    else { // para bloques en serie
      bloquesPorFila = (tamanio.width * 3 / 4) / (anchoBloque + entreBloquesX);
      if (bloquesPorFila == 0) {
        bloquesPorFila = 1;
        numFilas = numBloques;
      }
      else {
        numFilas = numBloques / bloquesPorFila;
        if (numBloques % bloquesPorFila != 0) {
          numFilas++;
        }
      }
      situarPuntosInicioSerie(puntoInicio, primerTerminal,
                              ultimoTerminal, bloquesPorFila, numFilas);
    }

    // dibujo los terminales si hay bloques
    if (numBloques > 0) {
      dibujaTerminales(g);
    }

    // dibujo las lneas y los bloques
    dibujaBloquesYLineas(g, bloquesPorFila, numFilas);

    // sito los mrgenes del diagrama para controlar el Scroll, si el nmero
    // de bloques es cero, no hace falta calcular nada.
    if (numBloques != 0) {
      if (paralelo) {
        puntoLimite = new Point(ultimoTerminal.x + anchoTerminal,
                                puntoInicio.y);
      }
      else {
        puntoLimite = new Point(ultimoTerminal.x + anchoTerminal +
                                margenScroll / 4,
                                ultimoTerminal.y + anchoTerminal + margenScroll);

      }
    }
    else {
      puntoLimite = new Point(0, 0);

    }
    setPreferredSize(new Dimension(puntoLimite.x, puntoLimite.y));

    revalidate();

    Iterator iterador = modelosFiguraBloque.iterator();
    while (iterador.hasNext()) {
      //aado a cada FiguraBloque el diagrama observador
      ( (FiguraBloque) (iterador.next())).addObserver(this);
    }

    ventanaPrincipal.actualizaMenus();
    actualizaPopups();
  } // fin paint

  /* Funcin que Devuelve el tamao del JScroll en el que se encuentra el
     diagrama, en caso de que no haya diagrama devuelve la dimensin 0,0 */
  public Dimension Tamanio() {
    Dimension tamanio;
    if (interfaz.FramePrincipal.diagramaScroll != null) {
      JScrollPane panelContenedor = interfaz.FramePrincipal.diagramaScroll;
      tamanio = panelContenedor.getSize();
    }
    else {
      tamanio = new Dimension(0, 0);

    }
    return tamanio;
  }

  public void cambiarDisposicionBloques() {
    Bloque nuevoBloque;
    if (!paralelo) {
      nuevoBloque = new BloqueParalelo(modelo.nombre(), modelo.padre());
    }
    else {
      nuevoBloque = new BloqueSerie(modelo.nombre(), modelo.padre());

    }
    for (int i = 0; i < modelo.componentes.size(); i++) {
      nuevoBloque.add( (Bloque) modelo.componentes.get(i), i);

    }
    modelo.deleteObservers();
    nuevoBloque.cambiarNombreSinComprobacion(modelo.nombre());
    nuevoBloque.addObserver(this);
    if (modelo.padre() != null) {
      int pos = modelo.padre().componentes.indexOf(modelo);
      modelo.padre().eliminaBloque(pos);
      modelo.padre().add(nuevoBloque, pos);
    }

    modelo = nuevoBloque;

    if (nuevoBloque.padre() == null) {
      ventanaPrincipal.cambiarDisposicionBloquePadre(nuevoBloque);
    }
  }

  private void situarPuntosInicioParalelo(Point puntoInicio,
                                          Point primerTerminal,
                                          Point ultimoTerminal) {
    /* Situamos la anchura de acuerdo al JScroll para que varie con el, margen
     para que el rea visible no sea pisada por las barras de Scroll*/
    int margen = 20;

    /* El tamao del Diagrama ser el del scroll, en caso de ser menor el ancho
      que el de dos bloques, el ancho de dos bloques se pondr como mnimo*/
    int anchuraVentana = Tamanio().width - margen;
    if (anchuraVentana < 2 * anchoBloque) {
      anchuraVentana = 2 * anchoBloque;
    }

    int tamanioDibujo = (numBloques * (altoBloque + entreBloquesY)
                         - entreBloquesY);
    int alturaVentana = 2 * margen + tamanioDibujo;

    /*Si es menor que el tamao que ocupa el Scroll, le damos al menos ese
         tamao*/
    if (alturaVentana < Tamanio().height) {
      alturaVentana = Tamanio().height;

    }
    puntoInicio.y = (alturaVentana / 2) -
        ( (numBloques * (altoBloque + entreBloquesY) - entreBloquesY) / 2);
    puntoInicio.x = ( (anchuraVentana - anchoBloque) / 2);
    primerTerminal.x = 5;
    primerTerminal.y = (alturaVentana / 2) - (anchoTerminal / 2);
    ultimoTerminal.x = ( (anchuraVentana - 5) - anchoTerminal);
    ultimoTerminal.y = ( (alturaVentana / 2) - (anchoTerminal / 2));

  }

  private void situarPuntosInicioSerie(Point puntoInicio,
                                       Point primerTerminal,
                                       Point ultimoTerminal,
                                       int bloquesPorFila, int numFilas) {
    // Calculo cunto tiene que ocupar el dibujo
    int alturaDibujo = 0;
    if (numFilas != 0) {
      alturaDibujo = numFilas * altoBloque + (numFilas - 1) * entreFilas;

      /* Es necesario aplicar unos margenes por esttica del dibujo y para que
       parte del dibujo no quede bajo la barra de scroll*/
    }
    int margenVertical = 20; // Superior y horizontal
    int margenHorizontal = 20;
    int alturaVentana = 2 * margenVertical + alturaDibujo;

    /*Si es menor que el tamao que ocupa el Scroll, le damos al menos ese
     tamao*/
    if (alturaVentana < Tamanio().height) {
      alturaVentana = Tamanio().height;

      /*Se sita el margen en caso de tomar como tamao el del scroll, y se
       da cmo tamao mnimo dos veces el del ancho del bloque*/
    }
    int anchuraVentana = Tamanio().width - margenHorizontal;
    if (anchuraVentana < 2 * anchoBloque) {
      anchuraVentana = 2 * anchoBloque;
    }

    setPreferredSize(new Dimension(anchuraVentana, alturaVentana));
    revalidate();

    puntoInicio.y = alturaVentana / 2 - alturaDibujo / 2;
    puntoInicio.x = anchuraVentana / 2 -
        ( (entreBloquesX + anchoBloque) * bloquesPorFila) / 2;

    // Sito los terminales
    primerTerminal.x = 5;
    primerTerminal.y = puntoInicio.y + (altoBloque / 2) - (anchoTerminal / 2);

    ultimoTerminal.x = anchuraVentana - (5 + anchoTerminal);

    ultimoTerminal.y = puntoInicio.y +
        (numFilas - 1) * (altoBloque + entreFilas) +
        altoBloque / 2 - anchoTerminal / 2;

  }

  public void dibujaTerminales(Graphics g) {
    g.fillRect(primerTerminal.x, primerTerminal.y, anchoTerminal,
               anchoTerminal);
    g.fillRect(ultimoTerminal.x, ultimoTerminal.y, anchoTerminal,
               anchoTerminal);
  }

  public void dibujaBloquesYLineas(Graphics g, int bloquesPorFila,
                                   int numFilas) {
    // Actualiza el estado de la variable reparable
    reparable = ventanaPrincipal.opcionesResolucion.esReparable();

    int filasDibujadas = 0;
    Iterator iterador = listaBloques.iterator();

    for (int i = 0; i < modelo.componentes.size(); i++) {
      Bloque bloqueEvaluado = (Bloque) iterador.next();
      boolean ext = bloqueEvaluado.esExtensible();

      // dibujo el bloque
      FiguraBloque figuraaDibujar = (FiguraBloque) modelosFiguraBloque.get(i);
      figuraaDibujar.dibujar(puntoInicio,
                             altoBloque, anchoBloque, g, espacioExtensibles,
                             reparable);
      // Dibujo las lneas
      if (paralelo) {
        /* Margen para los bloques en serie, que son ms anchos */
        int margen = 0;
        if (figuraaDibujar.modelo().esExtensible()
            && !figuraaDibujar.modelo().esParalelo()) {
          margen = FramePrincipal.prefVars.obtenerEspacioExtensibles();
        }
        g.drawLine( ( (anchoTerminal / 2) + primerTerminal.x),
                    (anchoTerminal / 2) + primerTerminal.y,
                    puntoInicio.x - margen, (puntoInicio.y + altoBloque / 2));
        g.drawLine( (anchoTerminal / 2 + ultimoTerminal.x),
                    (ultimoTerminal.y + (anchoTerminal / 2)),
                    puntoInicio.x + anchoBloque + margen,
                    (puntoInicio.y + altoBloque / 2));
      }
      else { // en serie
        /* Margen para los bloques en serie, que son ms anchos */
        int margen = 0;
        if (figuraaDibujar.modelo().esExtensible()
            && !figuraaDibujar.modelo().esParalelo()) {
          margen = FramePrincipal.prefVars.obtenerEspacioExtensibles();
        }

        FiguraBloque figuraSiguiente = null; // Para determinar el fin de lnea
        boolean siguienteSerie = false;      // Los bloques serie son ms anchos
        if (i < numBloques - 1)
          figuraSiguiente = (FiguraBloque) modelosFiguraBloque.get(i+1);

        // Para evitar que las lneas entre en las figuras serie
        int margenDcha = 0;
        if (figuraSiguiente != null
            && figuraSiguiente.modelo().esExtensible()
            && !figuraSiguiente.modelo().esParalelo()) // Figuras serie
          margenDcha = FramePrincipal.prefVars.obtenerEspacioExtensibles();

        // Lnea que une el primer terminal con la primera figura
        if (i == 0) {
          g.drawLine(primerTerminal.x + anchoTerminal / 2,
                     primerTerminal.y + anchoTerminal / 2,
                     puntoInicio.x - margen, puntoInicio.y + altoBloque / 2);
        }

        // Lneas intermedias que no son final de fila
        if ( (i != numBloques - 1) && (0 != (i + 1) % bloquesPorFila)) {
          g.drawLine(puntoInicio.x + anchoBloque + margen,
                     puntoInicio.y + (altoBloque / 2),
                     puntoInicio.x + anchoBloque + entreBloquesX - margenDcha,
                     puntoInicio.y + (altoBloque / 2));
        }

        // Lneas intermedias finales de fila
        // Utilizo anchoTerminal*2 como distancia entre el bloque y el vertice
        // del nexo
        if ( (0 == (i + 1) % bloquesPorFila) && (i != numBloques - 1)) {
          g.drawLine(puntoInicio.x + anchoBloque + margen,
                     puntoInicio.y + (altoBloque / 2),
                     puntoInicio.x + anchoBloque + (2 * anchoTerminal),
                     puntoInicio.y + (altoBloque / 2));
          g.drawLine(puntoInicio.x + +anchoBloque + (2 * anchoTerminal),
                     puntoInicio.y + (altoBloque / 2),
                     puntoInicio.x + +anchoBloque + (2 * anchoTerminal),
                     puntoInicio.y + altoBloque + (entreFilas / 2));
          g.drawLine(puntoInicio.x -
                     ( (anchoBloque + entreBloquesX) * (bloquesPorFila - 1)) -
                     2 * anchoTerminal,
                     puntoInicio.y + altoBloque + (entreFilas / 2),
                     puntoInicio.x + +anchoBloque + (2 * anchoTerminal),
                     puntoInicio.y + altoBloque + (entreFilas / 2));
          g.drawLine(puntoInicio.x -
                     ( (anchoBloque + entreBloquesX) * (bloquesPorFila - 1)) -
                     2 * anchoTerminal,
                     puntoInicio.y + altoBloque + (entreFilas / 2),
                     puntoInicio.x -
                     ( (anchoBloque + entreBloquesX) * (bloquesPorFila - 1)) -
                     2 * anchoTerminal,
                     puntoInicio.y + altoBloque + entreFilas + altoBloque / 2);
          g.drawLine(puntoInicio.x - margenDcha -
                     ( (anchoBloque + entreBloquesX) * (bloquesPorFila - 1)),
                     puntoInicio.y + altoBloque + entreFilas + altoBloque / 2,
                     puntoInicio.x -
                     ( (anchoBloque + entreBloquesX) * (bloquesPorFila - 1)) -
                     2 * anchoTerminal,
                     puntoInicio.y + altoBloque + entreFilas + altoBloque / 2);
        }

        // ltima lnea
        if (i == numBloques - 1) {
          g.drawLine(ultimoTerminal.x + anchoTerminal / 2,
                     ultimoTerminal.y + anchoTerminal / 2,
                     puntoInicio.x + anchoBloque + margen,
                     puntoInicio.y + altoBloque / 2);
        }
      }

      // Calcular el inicio del siguiente bloque
      if (paralelo) {
        puntoInicio.y = (puntoInicio.y + altoBloque + entreBloquesY);
      }
      else {
        filasDibujadas = (i + 1) / bloquesPorFila;
        if ( ( (i + 1) % bloquesPorFila) == 0) { // Si es fin de fila
          puntoInicio.y = puntoInicio.y + entreFilas + altoBloque;
          puntoInicio.x = getPreferredSize().width / 2 -
              ( (entreBloquesX + anchoBloque) * bloquesPorFila) / 2;
        }
        else {
          puntoInicio.x = puntoInicio.x + anchoBloque + entreBloquesX;
        }
      }
    }
  }

  public void eliminaBloque() {
    int i = 0;
    i = modelosFiguraBloque.indexOf(figuraSeleccionada);
    UndoableEdit edit = new ElementoEliminarEdit(modelo, i, this,
                                                 ventanaPrincipal.opcionesResolucion);
    modelo.eliminaBloque(i);
    modelosFiguraBloque.remove(i);
    // Se notifica al escuchador de eventos de hacer deshacer
    interfaz.FramePrincipal.undoSupport_.postEdit(edit);
    figuraSeleccionada = null;
    repaint();
  }

  public void copiarBloque(Bloque bloque) {
    cortarCopiar = bloque; // Sealo el bloque cortado para el pegado especial
    puedePegar = true;
    copiarBloques(bloque, false);
  }

  public void cortarBloque(Bloque bloque) {
    cortarCopiar = bloque; // Sealo el bloque cortado para el pegado especial
    puedePegar = true;
    UndoableEdit edit = new EditarCortarEdit(bloque.padre(), bloque, this,
                                  ventanaPrincipal.opcionesResolucion);
    copiarBloques(bloque, true);
    FramePrincipal.undoSupport_.postEdit(edit);
  }

  public boolean puedePegar() {
    return puedePegar;
  }

  public void asignarPuedePegar(boolean puede) {
    puedePegar = puede;
  }

  public void importarBloque() {
    File directorio;
    File archivo;
    JFileChooser elegirArchivo = new JFileChooser();
    int respuesta = elegirArchivo.showOpenDialog(this);
    // Se ha pulsado el botn aceptar para salir
    if (respuesta == JFileChooser.APPROVE_OPTION) {
      archivo = elegirArchivo.getSelectedFile();
      try {
        Bloque antiguoCopiar = null;
        if (puedePegar) {
          ObjectInputStream archivoCortar = new ObjectInputStream
              (new FileInputStream(archivoCopiarCortar));
          antiguoCopiar = (Bloque) archivoCortar.readObject();
          archivoCortar.close();
        }
        ObjectInputStream archivoAbrir = new ObjectInputStream
            (new FileInputStream(archivo));
        ObjectOutputStream archivoCortarb = new ObjectOutputStream
            (new FileOutputStream(archivoCopiarCortar));
        archivoCortarb.writeObject( (Bloque) archivoAbrir.readObject());
        archivoAbrir.close();
        archivoCortarb.close();
        this.pegarBloque(modelo, modelo.padre(), IMPORTADO);

        ObjectOutputStream archivoCortarc = new ObjectOutputStream
            (new FileOutputStream(archivoCopiarCortar));
        archivoCortarc.writeObject(antiguoCopiar);
        archivoCortarc.close();
      }
      catch (Exception e) {
        JOptionPane.showMessageDialog(null,
            "No se ha podido importar el contenido del archivo " +
            archivo,
            "Error al importar el fichero",
            JOptionPane.ERROR_MESSAGE);
      }
    }
  }

  public void pegarBloque(Bloque destino, Bloque padreDestino, int tipo) {
    boolean copiaSobreNuevo = false;
    int posicion;
    Bloque padreBloquePegado;

    if (!destino.esExtensible()) {
      posicion = padreDestino.componentes.indexOf(destino);
      padreBloquePegado = padreDestino;
    } else {
      posicion = destino.componentes.size();
      padreBloquePegado = destino;
      copiaSobreNuevo = true;
    }

    try {
      ObjectInputStream archivoPegar = new ObjectInputStream
          (new FileInputStream(archivoCopiarCortar));

      // Se lee el bloque guardado y se cierra el fichero
      Bloque bloqueNuevo = (Bloque) archivoPegar.readObject();
      archivoPegar.close();
      UndoableEdit edit = new EditarPegarEdit(destino, bloqueNuevo,  this,
                                              copiaSobreNuevo, posicion,
                                              padreBloquePegado);
      if (!destino.esExtensible()) {  // Slo cuando sobreescribimos
        padreDestino.eliminaBloque(posicion);
        padreDestino.add(bloqueNuevo, posicion);
      }  else {
        destino.add(bloqueNuevo, posicion);
      }
      FramePrincipal.undoSupport_.postEdit(edit);
      if (tipo == PEGADO) {
        asignarNombreAdecuado(bloqueNuevo);
        if (bloqueNuevo.esExtensible()) {
          for (int i = 0; i < bloqueNuevo.componentes.size(); i++) {
            asignarNombreAdecuado( (Bloque) bloqueNuevo.componentes.get(i));
          }
        }
      }
      else {
        ajustarNombresBloqueImportado(bloqueNuevo);
      }

      /* Si se ha copiado sobre el bloque que se est mostrando, repintar */
      if (padreDestino == null || padreDestino == modelo || destino == modelo)
        cambiarBloqueMostrado(modelo);
    }
    catch (Exception exc) {
      JOptionPane.showMessageDialog(null,
                                      "Se ha producido un error al pegar el "+
                                      "bloque "+ destino.nombre(),
                                      "Error al pegar",
                                      JOptionPane.ERROR_MESSAGE);
    } // Pasar a messageBox
  }

  /**Cambia los nombres de los bloques importados para que no haya nombres
   * de bloque repetidos. Si no hay otro bloque de igual nombre deja el nombre
   * original del bloque, sino le aade el apellido importado
   * @param importado Nombre del bloque al que se ha de revisar el nombre*/
  private void ajustarNombresBloqueImportado(Bloque importado) {
    String nuevoNombre = importado.nombre();
    // Nombre temporal para que el nombre real del bloque no interfiera en el
    // clculo
    importado.cambiarNombreSinComprobacion("WWWWWWWWWWWWWWWWWWWWW");
    if (!importado.nombreValido(nuevoNombre))
      nuevoNombre = nuevoNombre + " importado";
    int i = 2;
     String nombreAux = nuevoNombre;
    while (!importado.nombreValido(nuevoNombre)) {
      nuevoNombre = nombreAux + " "+i;
      i++;
    }
    importado.cambiarNombreSinComprobacion(nuevoNombre);
    if (importado.esExtensible()) {
      Iterator iterador = importado.componentes.iterator();
      while (iterador.hasNext()) {
        ajustarNombresBloqueImportado( (Bloque) iterador.next());
      }
    }
  }

  /**Funcin que quita todos los observadores del bloque pasado como parmetro
   * as como de sus hijos*/
  private void quitarObservadores(Bloque bloque) {
    bloque.deleteObservers();
    if (bloque.esExtensible()) {
      for (int i = 0; i < bloque.componentes.size(); i++) {
        quitarObservadores( (Bloque) bloque.componentes.get(i));
      }
    }
  }

  public void copiarBloques(Bloque origen, boolean cortado) {
// OJO: Esta llamada a la funcin quitarObservadores se elimina porque
// haca que los nodos del rbol dejasen de observar al nodo. Sin embargo,
// al eliminarla queda la figura vieja (que ya no se muestra) como observador
// del bloque, con lo cual no se libera la memoria y se est creando un leak
//    quitarObservadores(origen);
    try {
      ObjectOutputStream archivoCortar = new ObjectOutputStream
          (new FileOutputStream(archivoCopiarCortar));
      archivoCortar.writeObject(origen);
      archivoCortar.close();
    }
    catch (Exception exc) {
      System.out.println("Error interno del programa en copiarBloques");
    }
    if (cortado == true) {
      int posicion = origen.padre().componentes.indexOf(origen);
      origen.padre().eliminaBloque(posicion);
      cambiarBloqueMostrado(origen.padre());
    }
  }

  public void pegadoEspecial(Bloque destino, Bloque padreDestino) {
    boolean error = false;
    Bloque bloqueActual; // ndice para recorrer bloques

    // Comprobar que el bloque de destino no es el de origen o padre del de
    // origen
    bloqueActual = destino;
    while (bloqueActual != null && !error) {
      if (bloqueActual.nombre().compareTo(cortarCopiar.nombre()) == 0) {
        error = true;
      }
      bloqueActual = bloqueActual.padre();
    }
    if (!error) {
      pegadoEspecialBloques(destino, cortarCopiar, padreDestino);
      figuraSeleccionada = null;
    }
    else {
      JOptionPane.showMessageDialog(null,
                                    "No se puede realizar el pegado especial "
                                    + "cuando \nel bloque a pegar"
                                    + " forma parte del bloque destino",
                                    "Error de pegado especial",
                                    JOptionPane.ERROR_MESSAGE);
    }
  }

  private void pegadoEspecialBloques(Bloque destino, Bloque origen,
                                     Bloque padreDestino) {
    int posicion;
    boolean copiaSobreNuevo = false;
    Bloque padreBloquePegado;

    if (destino.esExtensible()) { // Copiamos sin sobreescribir otro bloque
      posicion = destino.componentes.size();
      padreBloquePegado = destino;
      copiaSobreNuevo = true;
    }
    else { // Sobreescribimos el destino
      posicion = padreDestino.componentes.indexOf(destino);
      padreBloquePegado = padreDestino;
    }
    UndoableEdit edit = new EditarPegarEdit(destino, origen, this,
                                            copiaSobreNuevo, posicion,
                                            padreBloquePegado);
    if (!destino.esExtensible()) {
      padreDestino.eliminaBloque(posicion);
      padreDestino.add(origen, posicion);
    } else {
      destino.add(origen, posicion);
    }

    // notifica al escuchador de eventos de hacer deshacer
    interfaz.FramePrincipal.undoSupport_.postEdit(edit);

    /* Si se ha copiado sobre el bloque que se est mostrando, repintar */
    if (padreDestino == null || padreDestino == modelo || destino == modelo)
      cambiarBloqueMostrado(modelo);
}

  private void asignarNombreAdecuado(Bloque bloque) {
    String nombreAnt = bloque.nombre();
    String nombre = bloque.nombre();
    bloque.cambiarNombreSinComprobacion("NOMBRE TEMPORAL");
    if (!bloque.nombreValido(nombre)) {
      nombre = "Copia de " + nombreAnt;
      int j = 2;
      while (!bloque.nombreValido(nombre)) {
        nombre = "Copia " + j + " de " + nombreAnt;
        j++;
      }
    }
    bloque.cambiarNombre(nombre);
  }

  public void bloqueExpansible(boolean tipo) {
    UndoableEdit edit = new ElementoBloqueExpansibleEdit(modelo, this);
    Bloque bloqueNuevo = modelo.bloqueExpansible(tipo);
    FiguraBloque nuevaFigura = new FiguraBloque(modelo, bloqueNuevo, this);
    modelosFiguraBloque.add(nuevaFigura);
    // notifico al escuchador de eventos de deshacer rehacer
    interfaz.FramePrincipal.undoSupport_.postEdit(edit);
  }

  public BloqueBasico bloqueSimple() {
    UndoableEdit edit = new ElementoBloqueSimpleEdit(modelo, this);
    Bloque bloqueNuevo = modelo.bloqueSimple();
    FiguraBloque nuevaFigura = new FiguraBloque(modelo, bloqueNuevo, this);

    // Marco la ltima creada como seleccionada
    Diagrama.figuraSeleccionada = nuevaFigura;
    modelosFiguraBloque.add(nuevaFigura);

    // notifico al escuchador de eventos de deshacer rehacer
    interfaz.FramePrincipal.undoSupport_.postEdit(edit);
    return (BloqueBasico) bloqueNuevo;
  }

  public void cambiarBloqueMostrado(Bloque nuevoBloque) {
    modelo = nuevoBloque;
    listaBloques = modelo.componentes;
    paralelo = modelo.esParalelo();
    modelosFiguraBloque.clear();
    numBloques = listaBloques.size();
    figuraSeleccionada = null;
    Iterator iterador = listaBloques.iterator();
    while (iterador.hasNext()) {
      modelosFiguraBloque.add(new FiguraBloque(modelo,
                                               (Bloque) iterador.next(), this));
    }
    ventanaPrincipal.gestionarCambioBloqueMostrado();
    repaint();
  }

  public void definirBloque(Bloque bloque) {
    if (bloque.esExtensible()) {
      return;
    }
    BloqueBasico bloqueaDefinir = (BloqueBasico) bloque;
    DefinirBloque dialogoDB =
        new DefinirBloque(ventanaPrincipal, bloqueaDefinir);
    Dimension Size = dialogoDB.getPreferredSize();
    Dimension getResDefSize = Tamanio();
    dialogoDB.setLocation(getResDefSize.width / 2,
                          getResDefSize.height / 2);
    dialogoDB.setModal(true);
    dialogoDB.pack();
    dialogoDB.setSize(340, 330);
    dialogoDB.show();
  }

  public Bloque obtenerBloqueMostrado() {
    return modelo;
  }

  public void subirNivel() {
    cambiarBloqueMostrado(modelo.padre());
  }

  public boolean puedeSubirNivel() {
    return (modelo.tienePadre());
  }

  private void elegirAnchoLineas(Graphics g) {
    float tamanioLineas = FramePrincipal.prefVars.obtenerAnchoLineas();
    Graphics2D g2 = (Graphics2D) g;
    BasicStroke trazo = new BasicStroke(tamanioLineas);
    g2.setStroke(trazo);
  }

  private void elegirTamanioBloques() {
    anchoBloque = FramePrincipal.prefVars.obtenerAnchoBloques();
    altoBloque = FramePrincipal.prefVars.obtenerAltoBloques();
    entreBloquesX = FramePrincipal.prefVars.obtenerEntreBloquesX();
    entreBloquesY = FramePrincipal.prefVars.obtenerEntreBloquesY();
    entreFilas = FramePrincipal.prefVars.obtenerEntreFilas();
    anchoTerminal = FramePrincipal.prefVars.obtenerAnchoTerminal();
    espacioExtensibles = FramePrincipal.prefVars.obtenerEspacioExtensibles();
    margenScroll = FramePrincipal.prefVars.obtenerMargenScroll();
  }

  private void prepararMenusPopup() {

    ActionListener cambiarNombreListener = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        DialogoCambiarNombre dialogoCN =
            new DialogoCambiarNombre(ventanaPrincipal, figuraSeleccionada);
        Dimension Size = dialogoCN.getPreferredSize();
        Dimension getResDefSize = Tamanio();
        Point loc = diagramaVisible.getLocation();
        dialogoCN.setLocation( (getResDefSize.width / 2) + loc.x,
                              (getResDefSize.height / 2) + loc.y);
        dialogoCN.setModal(true);
        dialogoCN.pack();
        dialogoCN.setSize(285, 150);
        dialogoCN.show();
      }
    };

    ActionListener eliminarBloqueListener = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        eliminaBloque();
      }
    };

    ActionListener definirBloqueListener = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        definirBloque(figuraSeleccionada.modelo());
      }
    };

    ActionListener cortarBloqueListener = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        cortarBloque(figuraSeleccionada.modelo());
      }
    };

    ActionListener copiarBloqueListener = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        copiarBloque(figuraSeleccionada.modelo());
      }
    };

    ActionListener pegarBloqueListener = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        Bloque destino;
        Bloque padre;
        if (figuraSeleccionada == null) {
          destino = modelo; // Copiamos sobre el bloque en el que estamos
          // BUG: Lo siguiente puede dar problemas con bloques especiales que estn
          // pegados en distintos niveles y, por lo tanto, tengan distinto padre
          padre = modelo.padre();
        }
        else {
          destino = figuraSeleccionada.modelo();
          padre = modelo;
        }
        pegarBloque(destino, padre, PEGADO);
      }
    };

    ActionListener pegadoEspecialBloqueListener = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        Bloque destino;
        Bloque padre;
        if (figuraSeleccionada == null) {
          destino = modelo; // Copiamos sobre el bloque en el que estamos
          // BUG: Lo siguiente puede dar problemas con bloques especiales que estn
          // pegados en distintos niveles y, por lo tanto, tengan distinto padre
          padre = modelo.padre();
        }
        else {
          destino = figuraSeleccionada.modelo();
          padre = modelo;
        }
        pegadoEspecial(destino, padre);
      }
    };

    ActionListener preferenciasListener = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        ventanaPrincipal.jMenuEditPreferences_actionPerformed(e);
      }
    };

    JMenuItem cambiarNombre = new JMenuItem("Cambiar Nombre");
    cambiarNombre.addActionListener(cambiarNombreListener);
    popFiguraBloque.add(cambiarNombre);
    definirBloque = new JMenuItem("Definir Bloque");
    definirBloque.addActionListener(definirBloqueListener);
    popFiguraBloque.add(definirBloque);
    popFiguraBloque.addSeparator();
    JMenuItem eliminarBloque = new JMenuItem("Eliminar Bloque");
    eliminarBloque.addActionListener(eliminarBloqueListener);
    popFiguraBloque.add(eliminarBloque);
    popFiguraBloque.addSeparator();
    JMenuItem cortar = new JMenuItem("Cortar");
    cortar.addActionListener(cortarBloqueListener);
    popFiguraBloque.add(cortar);
    JMenuItem copiar = new JMenuItem("Copiar");
    copiar.addActionListener(copiarBloqueListener);
    popFiguraBloque.add(copiar);
    pegar = new JMenuItem("Pegar");
    pegar.addActionListener(pegarBloqueListener);
    popFiguraBloque.add(pegar);
    pegadoEspecial = new JMenuItem("Pegado especial");
    pegadoEspecial.addActionListener(pegadoEspecialBloqueListener);
    popFiguraBloque.add(pegadoEspecial);

    pegarNoFigura = new JMenuItem("Pegar");
    pegarNoFigura.addActionListener(pegarBloqueListener);
    popNoFiguraBloque.add(pegarNoFigura);
    pegadoEspecialNoFigura = new JMenuItem("Pegado especial");
    pegadoEspecialNoFigura.addActionListener(pegadoEspecialBloqueListener);
    popNoFiguraBloque.add(pegadoEspecialNoFigura);
    popNoFiguraBloque.addSeparator();
    JMenuItem preferencias = new JMenuItem("Preferencias");
    preferencias.addActionListener(preferenciasListener);
    popNoFiguraBloque.add(preferencias);

    actualizaPopups();
  }

  private void actualizaPopups() {
    definirBloque.setEnabled( (figuraSeleccionada != null) &&
                             ! ( ( (Bloque) (figuraSeleccionada.modelo())).
                                esExtensible()));
    pegar.setEnabled(puedePegar());
    pegarNoFigura.setEnabled(puedePegar());
    pegadoEspecial.setEnabled(puedePegar());
    pegadoEspecialNoFigura.setEnabled(puedePegar());
  }

  /**Muestra el men pop up de los FiguraBloque*/
  void mostrarMenuPopup(MouseEvent e) {
    popFiguraBloque.show( ( (JPanel) e.getComponent()),
                         e.getX(), e.getY());
  }

  /**Muestra el men pop up del diagrama*/
  void mostrarMenuPopupNoFigura(MouseEvent e) {
    popNoFiguraBloque.show( ( (JPanel) e.getComponent()),
                           e.getX(), e.getY());
  }

  public void gestionarCambioNombre(String nombreAntiguo, String nombreActual) {
    ventanaPrincipal.gestionarCambioNombre(nombreAntiguo, nombreActual);
  }

  boolean cambiarSeleccionada(MouseEvent e) {
    Point puntoPinchado = new Point(e.getX(), e.getY());
    Iterator iteradorFiguraBloques = modelosFiguraBloque.iterator();
    int i = 0;
    while (iteradorFiguraBloques.hasNext()) {
      i++;
      FiguraBloque figuraEvaluada = (FiguraBloque) iteradorFiguraBloques.next();
      if (figuraEvaluada.pertenece(puntoPinchado)) {
        figuraSeleccionada = figuraEvaluada;
        repaint();
        return true;
      }
    }
    // Se ha pinchado fuera luego no habr figura seleccionada
    figuraSeleccionada = null;
    repaint();
    return false;
  }

  class EscuchadorMenusPopup
      extends MouseAdapter {
    public void MousePressed(MouseEvent e) {
      // Si cambiamos la figuraSeleccionada es que hemos pinchado en otra caja
      if (e.isPopupTrigger()) {
        if (cambiarSeleccionada(e)) {
          mostrarMenuPopup(e);
        }
        else {
          mostrarMenuPopupNoFigura(e);
        }
      }
    }

    public void mouseReleased(MouseEvent e) {
      if (e.isPopupTrigger()) {
        // Si cambia la figuraSeleccionada es se ha pinchado en otra caja
        if (cambiarSeleccionada(e)) {
          mostrarMenuPopup(e);
        }
        else {
          mostrarMenuPopupNoFigura(e);
        }
      }
      else if (e.getClickCount() >= 2) { // Doble click o mas
        // Si cambia la figuraSeleccionada es que se ha pinchado en otra caja
        if (cambiarSeleccionada(e)) {

          // Si la figura seleccionada no es extensible no tiene sentido el cambio
          if (figuraSeleccionada.modelo().esExtensible()) {
            cambiarBloqueMostrado(figuraSeleccionada.modelo());
          } else {
            // Si no estamos en una figura extensible, ser entonces un bloque
            // simple. Con doble clic se define
            definirBloque(figuraSeleccionada.modelo());
          }
        }
      }
      if (e.getClickCount() < 2) { // click simple
        cambiarSeleccionada(e);
      }
    }
  }

  private void incorporarAccionesTeclado() {
    Action accionEliminar = new AccionEliminarBloque(this);
    this.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(KeyStroke.
        getKeyStroke(127, 0), "accionEliminar");
    this.getActionMap().put("accionEliminar", accionEliminar);
  }
}

class AccionEliminarBloque
    extends AbstractAction {
  Diagrama diagrama;
  public AccionEliminarBloque(Diagrama diagrama) {
    this.diagrama = diagrama;
  }

  public void actionPerformed(ActionEvent e) {
    diagrama.eliminaBloque();
  }
}
