package interfaz;

import interfaz.deshacer.ElementoCambiarNombreEdit;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Point;
import java.text.DecimalFormat;
import java.util.Observable;

import javax.swing.JTextField;
import javax.swing.undo.UndoableEdit;

import bloques.Bloque;

/**
 * <p>Title: FiguraBloque.java</p>
 * <p>Description: Figuras que representan a los bloques en el diagrama</p>
 * @author Diego Daz Garca
 * @version 1.0
 */

public class FiguraBloque
    extends java.util.Observable
    implements
    java.util.Observer {

  private Bloque modelo;
  private int anchoBloque;
  private int altoBloque;
  Bloque bloquePadre;
  Graphics g;
  Point puntoInicio = new Point(0, 0);
  JTextField campoNombre = new JTextField();
  Diagrama diagrama;
  int parametroMostrado;
  int parametroMostradoReparable;
  int decimales;

  /**Constructor
   * @param bloquePadre_ Bloque principal
   * @param model_ Bloque al que representa
   * @param diagrama_ Diagraama en el que se dibujan los FiguraBloque*/
  public FiguraBloque(Bloque bloquePadre_, Bloque model_, Diagrama diagrama_) {

    modelo = model_;
    modelo.addObserver(this);
    bloquePadre = bloquePadre_;
    diagrama = diagrama_;
    // Al crearla la marca como la figura seleccionada
    diagrama.figuraSeleccionada = this;
  }

  /**Dibuja el FiguraBloque en el diagrama
   * @param puntoInicio_ Punto inicial desde el que empezar a dibujar
   * @param altoBloque_ Alto en pxeles del bloque
   * @param anchoBloque_ Ancho en pxeles del bloque
   * @param g_ Sistema de grficos en el que dibujar
   * @param espExtensibles Espacio entre los dos bloques que representan un
   * extensible
   * @param reparable Carcter de reparable o no del diagrama*/
  public void dibujar(Point puntoInicio_, int altoBloque_, int anchoBloque_,
                      Graphics g_, int espExtensibles, boolean reparable) {
    decimales = FramePrincipal.prefVars.obtenerNumeroDecimalesPantalla();
    g = g_;
    altoBloque = altoBloque_;
    anchoBloque = anchoBloque_;
    puntoInicio.x = puntoInicio_.x;
    puntoInicio.y = puntoInicio_.y;
    boolean extensible = modelo.esExtensible();

    int anchoLinea = interfaz.FramePrincipal.prefVars.obtenerAnchoLineas();

    // Cambia al color de los bloques y dibuja el interior de los bloques
    Color colorBloques = interfaz.FramePrincipal.prefVars.obtenerColorBloques();
    Color colorSeleccionada = crearColorSeleccionada(colorBloques);
    if (figuraSeleccionada()) {
      g.setColor(colorSeleccionada);
    }
    else {
      g.setColor(colorBloques);
    }
    g.fillRect(puntoInicio.x, puntoInicio.y,
               anchoBloque, altoBloque);

    // Cambia al color de las lneas y dibuja el exterior de los bloques
    g.setColor(interfaz.FramePrincipal.prefVars.obtenerColorLineas());
    if (extensible) {
      int tamSegmento = 8;
      if (modelo.esParalelo()) {
        // Los bloques en paralelo llevan una lnea por arriba y otra por abajo
        // y unos segmentos pequeos
        int xIni = puntoInicio.x - espExtensibles;
        int xFin = xIni + anchoBloque + 2 * espExtensibles;
        int y = puntoInicio.y - espExtensibles;

        // Parte de arriba
        g.drawLine(xIni, y, xFin, y);                // Lnea principal
        g.drawLine(xIni, y, xIni, y + tamSegmento);  // Esquina sup. izda.
        g.drawLine(xFin, y, xFin, y + tamSegmento);  // Esquina sup. dcha.

        // Parte de abajo
        y += altoBloque + 2 * espExtensibles;
        g.drawLine(xIni, y, xFin, y);                // Lnea principal
        g.drawLine(xIni, y, xIni, y - tamSegmento);  // Esquina inf. izda.
        g.drawLine(xFin, y, xFin, y - tamSegmento);  // Esquina inf. dcha.
      }
      else {
        // Los bloques en serie llevan una lnea por la dcha. y otra por la
        // izda. y unos segmentos pequeos en horizontal
        int x = puntoInicio.x - espExtensibles;
        int yIni = puntoInicio.y - espExtensibles;
        int yFin = yIni + altoBloque + 2 * espExtensibles;

        // Parte derecha
        g.drawLine(x, yIni, x, yFin);               // Lnea principal
        g.drawLine(x, yIni, x + tamSegmento, yIni); // Esquina sup. izda.
        g.drawLine(x, yFin, x + tamSegmento, yFin); // Esquina inf. izda.

        // Parte izquierda
        x += anchoBloque + 2 * espExtensibles;
        g.drawLine(x, yIni, x, yFin);
        g.drawLine(x, yIni, x - tamSegmento, yIni); // Esquina sup. dcha.
        g.drawLine(x, yFin, x - tamSegmento, yFin); // Esquina inf. dcha.
      }
    }
    g.drawRect(puntoInicio.x, puntoInicio.y, anchoBloque, altoBloque);

    escribeNombreBloque(g, puntoInicio, reparable);
  }

  /**Funcin para el modelo MVC que actualiza la vista
   * @param t Objeto observable que produjo la llamada
   * @param o Objeto que se puede pasar con el update*/
  public void update(Observable t, Object o) {
    setChanged();
    notifyObservers();
  }

  private void escribeNombreBloque(Graphics g, Point puntoInicio,
                                   boolean reparable) {
    parametroMostrado = FramePrincipal.prefVars.obtenerValorMostrarFiabilidad();
    parametroMostradoReparable =
        FramePrincipal.prefVars.obtenerValorMostrarReparabilidad();

    // Cambia el tamao de la fuente de acuerdo al de preferencias
    int tamanioFuente = interfaz.FramePrincipal.prefVars.obtenerTamanioFuente();
    Font fuente = new Font("Dialog", Font.PLAIN, tamanioFuente);
    g.setFont(fuente);
    // Almacena el color de g para reestablecerlo despues de escribir
    Color colorg = g.getColor();
    // Cambia el color de g de acuerdo al elegido para la fuente
    g.setColor(FramePrincipal.prefVars.obtenerColorFuente());
    g.drawString(modelo.nombre(), puntoInicio.x + anchoBloque / 32,
                 puntoInicio.y + 3 * altoBloque / 10);
    String cadenaMostrada = "";
    double valorMostrado = 0;
    String formato = "0";
    if (decimales > 0) {
      formato += ".";
    }
    for (int i = 0; i < decimales; i++) {
      formato += "#";
    }
    DecimalFormat df = new DecimalFormat(formato);
    if ( (modelo.comprobar(reparable).compareTo("") == 0)) {
      switch (parametroMostrado) {
        case 0:
          valorMostrado = modelo.obtenerLambda(reparable);
          cadenaMostrada = "Lambda = ";
          break;
        case 1:
          valorMostrado = modelo.obtenerMttf(reparable);
          cadenaMostrada = "MTTF = ";
          break;
        case 2:
          valorMostrado = modelo.obtenerMtbf(reparable);
          cadenaMostrada = "MTBF = ";
          break;
      }

      cadenaMostrada += df.format(valorMostrado);
      if (parametroMostrado != 3) {
        g.drawString(cadenaMostrada, puntoInicio.x + anchoBloque / 32,
                     puntoInicio.y + 7 * altoBloque / 10);
      }
    }
    if (modelo.comprobar(true).compareTo("") == 0) {
      switch (parametroMostradoReparable) {
        case 0:
          valorMostrado = modelo.obtenerMu();
          cadenaMostrada = "Mu = ";
          break;
        case 1:
          valorMostrado = modelo.obtenerMttr();
          cadenaMostrada = "MTTR = ";
          break;
      }
      cadenaMostrada += df.format(valorMostrado);
      if (parametroMostradoReparable != 2) {
        g.drawString(cadenaMostrada, puntoInicio.x + anchoBloque / 32,
                     puntoInicio.y + 19 * altoBloque / 20);

        // Reestablezco el color de g
      }
      g.setColor(colorg);
    }
  }

  private Color crearColorSeleccionada(Color colorBloques) {
    Color colorSeleccionada;
    int rojo = colorBloques.getRed() - 70;
    int verde = colorBloques.getGreen() - 70;
    int azul = colorBloques.getBlue() - 70;

    // Comprobacin de que no excedan el rango
    if (rojo < 0) {
      rojo = 0;
    }
    if (verde < 0) {
      verde = 0;
    }
    if (azul < 0) {
      azul = 0;

    }
    colorSeleccionada = new Color(rojo, verde, azul);
    return colorSeleccionada;
  }

  /**Cambia el nombre del bloque que representa el figuraBloque
   * @param nuevoNombre Nuevo nombre del bloque
   * @return Cierto si se ha cambiado el nombre falso en caso contrario*/
  public boolean cambiarNombre(String nuevoNombre) {
    boolean cambiado = true;
    String nombreAnt = modelo.nombre();

    if (modelo.cambiarNombre(nuevoNombre)) {
      UndoableEdit edit = new ElementoCambiarNombreEdit(diagrama, modelo,
          nombreAnt, nuevoNombre);
      diagrama.gestionarCambioNombre(nombreAnt, nuevoNombre);

      // notifico a los observadores
      setChanged();
      notifyObservers();
      // notifico al escuchador de eventos de deshacer rehacer
      interfaz.FramePrincipal.undoSupport_.postEdit(edit);
    }
    else {
      cambiado = false;
    }
    return cambiado;
  }

  /**Devuelve el nombre del bloque
   * @return El nombre del bloque que representa*/
  public String obtenerNombre() {
    return modelo.nombre();
  }

  private boolean figuraSeleccionada() {
    if (diagrama.figuraSeleccionada == this) {
      return true;
    }
    else {
      return false;
    }
  }

  /**Comprueba si un punto est dentro del FiguraBloque o no
   * @param puntoPinchado Es el punto que se desea comprobar
   * @return Cierto si el punto pertenece al FiguraBloque, falso en caso
   * contrario*/
  public boolean pertenece(Point puntoPinchado) {
    int diferenciax = puntoPinchado.x - puntoInicio.x;
    int diferenciay = puntoPinchado.y - puntoInicio.y;
    // si el punto est a la izquierda o encima de la Figura, no est dentro
    if ( (diferenciax < 0) || (diferenciay < 0)) {
      return false;
    }
    // si el punto est a la derecha del objeto o por debajo
    if ( (diferenciax > anchoBloque) || (diferenciay > altoBloque)) {
      return false;
    }
    // sino es que est dentro
    return true;
  }

  /**Observador del modelo del figurabloque
   * @return Devuelve el bloque al que observa el FiguraBloque*/
  public Bloque modelo() {
    return modelo;
  }
}
