package interfaz;

import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.StringSelection;
import java.awt.event.ActionEvent;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.Iterator;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.border.TitledBorder;

import bloques.Bloque;
import bloques.BloqueBasico;
import javax.swing.table.*;
import org.jfree.data.category.*;
import org.jfree.chart.*;
import org.jfree.chart.plot.*;
import org.jfree.data.xy.*;
import org.jfree.chart.renderer.category.*;
import org.jfree.chart.renderer.xy.*;
import java.awt.*;

/**
 * <p>Title: Resultados.java</p>
 * <p>Description: Clase para las ventanas en las que se mostrarn los
 * resultados
 * @author Diego Daz Garca
 * @version 1.0
 */

public class Resultados
    extends JFrame {
  private  static final int ARCHIVO = 0;
  private  static final int PORTAPAPELES = 1;
  private JPanel jPanel1 = new JPanel();
  private TitledBorder titledBorder1;
  private JButton botonCopiar = new JButton();
  private JButton botonCerrar = new JButton();
  private JButton botonExportar = new JButton();
  private JButton botonCrearGrafico = new JButton();
  private Opciones opcionesResolucion;
  private Bloque bloqueaResolver;

  // Datos de variar parmetros
  private String elemento = null; // al que pertenece el parmetro a variar
  private String parametro = null; // el que hay que variar
  private double inicial, ultimo, incremento;

  private double parametroTiempo;
  private JScrollPane jScrollPane1 = new JScrollPane();
  private boolean reparable = false;
  private BorderLayout borderLayout1 = new BorderLayout();
  private FlowLayout flowLayout1 = new FlowLayout();
  private int decimales = FramePrincipal.prefVars.obtenerNumeroDecimalesResultado();
  private boolean variarParametros; // Indica si se est haciendo un bucle
  private FramePrincipal ventanaPrincipal;

  // Tabla de resultados y sus datos
  private JTable tabla;
  private int numCols = 0;
  private int numFilas = 0;
  private int col = 0;
  private int fila = 0;
  private String[] nombresColumna;
  private Object[][] datos;

  // Resolucin sin variar parmetros
  public Resultados(Bloque bloqueaResolver, Opciones opcionesResolucion,
                    double tiempo, FramePrincipal ventanaPrincipal) {
    super("Resultados");
    this.ventanaPrincipal = ventanaPrincipal;
    this.opcionesResolucion = opcionesResolucion;
    this.bloqueaResolver = bloqueaResolver;
    this.reparable = opcionesResolucion.esReparable();
    parametroTiempo = tiempo;
    variarParametros = false;

    try {
      jbInit();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    if (comprobarBloques()) {
      generarResultados();
      insertarTablaResultados();
    }
  }

  // Resolucin variando parmetros
  public Resultados(Bloque bloqueaResolver, String parametro,
                    Opciones opcionesResolucion, String elemento,
                    BloqueBasico bloque, double inicial, double ultimo,
                    double incremento, FramePrincipal ventanaPrincipal) {
    super("Resultados");
    this.ventanaPrincipal = ventanaPrincipal;
    this.opcionesResolucion = opcionesResolucion;
    this.bloqueaResolver = bloqueaResolver;
    this.reparable = opcionesResolucion.esReparable();
    variarParametros = true;

    if (elemento == "Tiempo") {
      parametroTiempo = (new Double(inicial)).doubleValue();
    }
    else {
      parametroTiempo = opcionesResolucion.valorParametroTiempo().doubleValue();
    }

    this.elemento = elemento;
    this.parametro = parametro;
    this.inicial = inicial;
    this.ultimo = ultimo;
    this.incremento = incremento;

    try {
      jbInit();
    }
    catch (Exception e) {
      e.printStackTrace();
    }

    if (comprobarBloques()) {
      anadirIncrementos(bloque);
      insertarTablaResultados();
    }
  }

  private void anadirIncrementos(BloqueBasico bloque) {
    if (elemento == "Tiempo") {
      insertarResultado(parametroTiempo);
      generarResultados();
      while (parametroTiempo < ultimo) {
        parametroTiempo = parametroTiempo +
            (new Double(incremento)).doubleValue();
        insertarResultado(parametroTiempo);
        generarResultados();
      }
    }
    else {
      boolean definidoMTTF = bloque.definidoMTTF();
      // El valor anterior de lambda o MTTF se guarda para poder ser restaurado
      double valorAnterior;
      if (definidoMTTF) {
        valorAnterior = bloque.obtenerMttf(reparable);
      }
      else {
        valorAnterior = bloque.obtenerLambda(reparable);
      }
      boolean definidoMTTR = bloque.definidoMTTR();
      // Lo mismo para el valor de Mu o mttr
      double valorAnteriorRepar;
      if (definidoMTTR) {
        valorAnteriorRepar = bloque.obtenerMttr();
      }
      else {
        valorAnteriorRepar = bloque.obtenerMu();
      }
      if (parametro == "Tasa de fallos") {
        double inicio = inicial;
        bloque.asignarLambda(inicio);
        insertarResultado(bloque.obtenerLambda(reparable));
        generarResultados();
        while (inicio < ultimo) {
          inicio += incremento;
          bloque.asignarLambda(inicio);
          insertarResultado(bloque.obtenerLambda(reparable));
          generarResultados();
        }
      }

      else if (parametro == "MTTF") {
        double inicio = inicial;
        bloque.asignarMttf(inicio);
        insertarResultado(bloque.obtenerMttf(reparable));
        generarResultados();
        while (inicio < ultimo) {
          inicio += incremento;
          bloque.asignarMttf(inicio);
          insertarResultado(bloque.obtenerMttf(reparable));
          generarResultados();
        }
      }

      else if (parametro == "Tasa de reparacin") {
        double inicio = inicial;
        bloque.asignarMu(inicio);
        insertarResultado(bloque.obtenerMu());
        generarResultados();
        while (inicio < ultimo) {
          inicio += incremento;
          bloque.asignarMu(inicio);
          insertarResultado(bloque.obtenerMu());
          generarResultados();
        }
      }
      else if (parametro == "MTTR") {
        double inicio = inicial;
        bloque.asignarMu(inicio);
        insertarResultado(bloque.obtenerMttr());
        generarResultados();
        while (inicio < ultimo) {
          inicio += incremento;
          bloque.asignarMttr(inicio);
          insertarResultado(bloque.obtenerMttr());
          generarResultados();
        }
      }

      if (definidoMTTF) {
        bloque.asignarMttf(valorAnterior);
      }
      else {
        bloque.asignarLambda(valorAnterior);
      }
      if (definidoMTTR) {
        bloque.asignarMttr(valorAnteriorRepar);
      }
      else {
        bloque.asignarMu(valorAnteriorRepar);
      }
    } // Fin del else (parametro != tiempo)
  }

  private void jbInit() throws Exception {
    setIconImage(ventanaPrincipal.getIconImage());
    titledBorder1 = new TitledBorder("");
    this.setEnabled(true);
    this.setResizable(true);
    this.getContentPane().setLayout(borderLayout1);
    jPanel1.setLayout(flowLayout1);
    botonCopiar.setText("Copiar al portapapeles");
    botonCopiar.setFont(new java.awt.Font("Dialog", 0, 11));
    botonCopiar.addActionListener(new AccionExportar
                                  (PORTAPAPELES, ventanaPrincipal));
    botonCerrar.setFont(new java.awt.Font("Dialog", 0, 11));
    botonCerrar.setText("Cerrar");
    botonCerrar.addActionListener(new AccionCerrar());
    botonExportar.setFont(new java.awt.Font("Dialog", 0, 11));
    botonExportar.setText("Exportar a fichero");
    botonExportar.addActionListener(new AccionExportar
                                    (ARCHIVO, ventanaPrincipal));
    botonCrearGrafico.setText("Crear grfico");
    botonCrearGrafico.addActionListener(new AccionCrearGrafico());
    jScrollPane1.setHorizontalScrollBarPolicy(JScrollPane.
                                              HORIZONTAL_SCROLLBAR_AS_NEEDED);
    this.getContentPane().add(jPanel1, BorderLayout.SOUTH);
    jPanel1.add(botonExportar, null);
    jPanel1.add(botonCopiar, null);
    jPanel1.add(botonCrearGrafico, null);
    jPanel1.add(botonCerrar, null);
    this.getContentPane().add(jScrollPane1, BorderLayout.CENTER);
    inicializarTabla();
  }

  /**Funcion que recibe el nombre de un bloque y busca entre el otro parametro
   * bloque y sus componentes el bloque. Ha de existir el bloque cuyo nombre
   * buscamos, en caso contrario devolver null*/
  private Bloque encontrarBloque(String nombre, Bloque bloque) {
    if (bloque.nombre() == nombre) {
      return bloque;
    }
    Iterator iterador = bloque.componentes.iterator();
    Bloque bloqueDevuelto = null;
    while (iterador.hasNext()) {
      Bloque bloqueActual = (Bloque) iterador.next();

      if (bloqueActual.nombre() == nombre) {
        bloqueDevuelto = bloqueActual;

      }
      if (bloqueActual.esExtensible() && (bloqueActual.nombre() != nombre)) {
        bloqueDevuelto = encontrarBloque(nombre, bloqueActual);
        // si ya lo hemos encontrado
      }
      if (bloqueDevuelto != null) {
        break;
      }
    }
    return bloqueDevuelto;
  }

  /**Devuelve cierto si es posible calcular los datos pedidos en opciones*/
  private boolean comprobarBloques() {
    String correcto = "";
    correcto = bloqueaResolver.comprobar(reparable);
    if (correcto != "") {
      JOptionPane.showMessageDialog(null,
                                    correcto,
                                    "Error al resolver",
                                    JOptionPane.ERROR_MESSAGE);
    }
    return (correcto == "");
  }

  private void insertarResultado(double numero) {
    datos[fila][col] = Double.toString(numero);
    col++;
    if (col == numCols) {
      col = 0;
      fila++;
    }
  }

  private void insertarResultado(String cad) {
    datos[fila][col] = cad;
    col++;
    if (col == numCols) {
      col = 0;
      fila++;
    }
  }

  // La fiabilidad real slo se debe mostrar en sistemas no reparables porque
  // en los reparables no se puede calcular (hay que conformarse con la
  // equivalente)
  private boolean mostrarFiabilidad() {
    return opcionesResolucion.seleccionadoFiabilidad() && !reparable;
  }

  private void inicializarTabla() {
    Object[] listaNombresBloque =
        opcionesResolucion.obtenerNombreBloquesResolver();
    int numBloques = listaNombresBloque.length;

    int numOpciones = 0;
    if (opcionesResolucion.seleccionadoDisponibilidad()) {
      numOpciones++;
    }
    if (mostrarFiabilidad()) {
      numOpciones++;
    }
    if (opcionesResolucion.seleccionadoFiabilidadEq()) {
      numOpciones++;
    }
    if (opcionesResolucion.seleccionadoLambda()) {
      numOpciones++;
    }
    if (opcionesResolucion.seleccionadoMTBF()) {
      numOpciones++;
    }
    if (opcionesResolucion.seleccionadoMTTF()) {
      numOpciones++;
    }
    if (opcionesResolucion.seleccionadoMTTR()) {
      numOpciones++;
    }
    if (opcionesResolucion.seleccionadoMu()) {
      numOpciones++;

    }
    if (variarParametros) {
      // 1 para el parmetro
      numCols = 1 + numBloques * numOpciones;
      nombresColumna = new String[numCols];
      if (elemento == "Tiempo") {
        nombresColumna[0] = "Tiempo";
      }
      else {
        nombresColumna[0] = elemento + " (" + parametro + ")";
      }
      int col = 1;
      for (int i = 0; i < numBloques; i++) {
        String bloque = (String) listaNombresBloque[i];
        if (mostrarFiabilidad()) {
          nombresColumna[col++] = cabeceraBloque("Fiabilidad", bloque);
        }
        if (opcionesResolucion.seleccionadoFiabilidadEq()) {
          nombresColumna[col++] = cabeceraBloque("Fiabilidad Eq", bloque);
        }
        if (opcionesResolucion.seleccionadoDisponibilidad()) {
          nombresColumna[col++] = cabeceraBloque("Disponibilidad", bloque);
        }
        if (opcionesResolucion.seleccionadoLambda()) {
          nombresColumna[col++] = cabeceraBloque("Tasa de fallos", bloque);
        }
        if (opcionesResolucion.seleccionadoMu()) {
          nombresColumna[col++] = cabeceraBloque("Tasa de reparacin", bloque);
        }
        if (opcionesResolucion.seleccionadoMTTF()) {
          nombresColumna[col++] = cabeceraBloque("MTTF", bloque);
        }
        if (opcionesResolucion.seleccionadoMTTR()) {
          nombresColumna[col++] = cabeceraBloque("MTTR", bloque);
        }
        if (opcionesResolucion.seleccionadoMTBF()) {
          nombresColumna[col++] = cabeceraBloque("MTBF", bloque);
        }
      }
      numFilas = 1 + (int) ( (ultimo - inicial) / incremento);
      if ( ( (ultimo - inicial) % incremento) != 0) {
        numFilas++;
      }
    }
    else {
      // 1 para el nombre de bloque
      numCols = 1 + numOpciones;
      nombresColumna = new String[numCols];
      nombresColumna[0] = "Bloque";
      int col = 1;
      if (mostrarFiabilidad()) {
        nombresColumna[col++] = "Fiabilidad";
      }
      if (opcionesResolucion.seleccionadoFiabilidadEq()) {
        nombresColumna[col++] = "Fiabilidad Eq";
      }
      if (opcionesResolucion.seleccionadoDisponibilidad()) {
        nombresColumna[col++] = "Disponibilidad";
      }
      if (opcionesResolucion.seleccionadoLambda()) {
        nombresColumna[col++] = "Tasa de fallos";
      }
      if (opcionesResolucion.seleccionadoMu()) {
        nombresColumna[col++] = "Tasa de reparacin";
      }
      if (opcionesResolucion.seleccionadoMTTF()) {
        nombresColumna[col++] = "MTTF";
      }
      if (opcionesResolucion.seleccionadoMTTR()) {
        nombresColumna[col++] = "MTTR";
      }
      if (opcionesResolucion.seleccionadoMTBF()) {
        nombresColumna[col++] = "MTBF";
      }
      numFilas = numBloques;
    }

    datos = new Object[numFilas][numCols];
    col = 0;
    fila = 0;
  }

  private void insertarTablaResultados() {
    tabla = new JTable(datos, nombresColumna) {
      // Hacer que la tabla no sea editable
      public boolean isCellEditable(int rowIndex, int vColIndex) {
        return false;
      }
    };

    calcColumnWidths(tabla);

    // Mirar si la tabla es muy grande y necesita barras de scroll
    int ancho = 0;
    TableColumnModel colModel = tabla.getColumnModel();
    for (int i = 0; i < tabla.getColumnCount(); i++) {
      ancho += colModel.getColumn(i).getPreferredWidth();
    }
    if (ancho > 340) {
      // 340 es el nmero de pixels que hay para la ventana de resultados
      // por defecto. Si el tamao es mayor, quitar el autoresize porque impide
      // que haya scroll horizontal con lo que, cuando hay muchas columnas, no
      // se pueden ver todos los datos.
      tabla.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
    }

    jScrollPane1.getViewport().add(tabla, null);
    this.pack();
  }

  // Tomado de http://www.chka.de/swing/table/cell-sizes.html
  // Hace que que las columnas se adapten al tamao de su contenido
  public static void calcColumnWidths(JTable table) {
    JTableHeader header = table.getTableHeader();

    TableCellRenderer defaultHeaderRenderer = null;

    if (header != null) {
      defaultHeaderRenderer = header.getDefaultRenderer();

    }
    TableColumnModel columns = table.getColumnModel();
    TableModel data = table.getModel();

    int margin = columns.getColumnMargin(); // only JDK1.3

    int rowCount = data.getRowCount();

    int totalWidth = 0;

    for (int i = columns.getColumnCount() - 1; i >= 0; --i) {
      TableColumn column = columns.getColumn(i);

      int columnIndex = column.getModelIndex();

      int width = -1;

      TableCellRenderer h = column.getHeaderRenderer();

      if (h == null) {
        h = defaultHeaderRenderer;

      }
      if (h != null) { // Not explicitly impossible
        Component c = h.getTableCellRendererComponent
            (table, column.getHeaderValue(),
             false, false, -1, i);

        width = c.getPreferredSize().width;
      }

      for (int row = rowCount - 1; row >= 0; --row) {
        TableCellRenderer r = table.getCellRenderer(row, i);

        Component c = r.getTableCellRendererComponent
            (table,
             data.getValueAt(row, columnIndex),
             false, false, row, i);

        width = Math.max(width, c.getPreferredSize().width);
      }

      if (width >= 0) {
        column.setPreferredWidth(width + margin); // <1.3: without margin
      }
      else {
        ; // ???
      }

      totalWidth += column.getPreferredWidth();
    }
  }

  private String cabeceraBloque(String opcion, String nombreBloque) {
    return opcion + " (" + nombreBloque + ")";
  }

  /**Genera los resultados que se mostrarn por pantalla */
  private void generarResultados() {
    Object[] ListaNombresBloque =
        opcionesResolucion.obtenerNombreBloquesResolver();

    int tamanio = ListaNombresBloque.length;
    Bloque bloqueEvaluado = new BloqueBasico("", null);

    for (int i = 0; i < tamanio; i++) {
      bloqueEvaluado = encontrarBloque( (String) ListaNombresBloque[i],
                                       bloqueaResolver);
      if (bloqueEvaluado == null) {
        JOptionPane.showMessageDialog(null,
            "Se ha producido un error interno de la aplicacin: "
            + "no se ha encontrado el bloque " + (String) ListaNombresBloque[i],
                                      "Error interno al resolver",
                                      JOptionPane.ERROR_MESSAGE);
        throw new RuntimeException("bloque no encontrado " +
                                   (String) ListaNombresBloque[i]);
      }
      if (!variarParametros) {
        insertarResultado( (String) ListaNombresBloque[i]);
      }
      generarResultadosBloque(bloqueEvaluado);
    }
  }

  private void generarResultadosBloque(Bloque bloqueEvaluado) {
    boolean reparable = opcionesResolucion.esReparable();

    if (mostrarFiabilidad()) {
      insertarResultado(bloqueEvaluado.obtenerFiabilidad(parametroTiempo,
          reparable));
    }
    if (opcionesResolucion.seleccionadoFiabilidadEq()) {
      insertarResultado(bloqueEvaluado.obtenerFiabilidadEq(parametroTiempo,
          reparable));
    }
    if (opcionesResolucion.seleccionadoDisponibilidad()) {
      insertarResultado(bloqueEvaluado.obtenerDisponibilidad());
    }
    if (opcionesResolucion.seleccionadoLambda()) {
      insertarResultado(bloqueEvaluado.obtenerLambda(reparable));
    }
    if (opcionesResolucion.seleccionadoMu()) {
      insertarResultado(bloqueEvaluado.obtenerMu());
    }
    if (opcionesResolucion.seleccionadoMTTF()) {
      insertarResultado(bloqueEvaluado.obtenerMttf(reparable));
    }
    if (opcionesResolucion.seleccionadoMTTR()) {
      insertarResultado(bloqueEvaluado.obtenerMttr());
    }
    if (opcionesResolucion.seleccionadoMTBF()) {
      insertarResultado(bloqueEvaluado.obtenerMtbf(reparable));
    }
  }

  private String pasarResultadosaCadena() {
    // Utilizar un StringBuffer porque es ms rpido que un String
    StringBuffer buffer = new StringBuffer();
    int i, j;
    for (i = 0; i < numCols; i++) {
      buffer.append(nombresColumna[i] + "\t ");
    }
    for (i = 0; i < numFilas; i++) {
      System.out.println("Paso fila: " + i);
      buffer.append("\n");
      for (j = 0; j < numCols; j++) {
        buffer.append(datos[i][j] + "\t ");
      }
    }
    buffer.append("\n");
    return buffer.toString();
  }

  class AccionCerrar
      extends AbstractAction {
    public void actionPerformed(ActionEvent e) {
      dispose();
    }
  }

  class AccionExportar
      extends AbstractAction {
    int medio;
    FramePrincipal ventanaPrincipal;
    public AccionExportar(int medio, FramePrincipal ventanaPrincipal) {
      super();
      this.medio = medio;
      this.ventanaPrincipal = ventanaPrincipal;
    }

    public void actionPerformed(ActionEvent e) {
      if (medio == ARCHIVO) {
        guardarAArchivo();
      }
      else {
        copiarAPortapapeles();
      }
    }

    private void guardarAArchivo() {
      File archivo = null;
      try {
        archivo = ventanaPrincipal.elegirArchivoGuardar("txt",
            new FiltroTexto());
        if (archivo != null) {
          PrintWriter salida = new PrintWriter(new BufferedWriter
                                               (new FileWriter(archivo)));
          String cadenaaMostrar = pasarResultadosaCadena();
          salida.print(cadenaaMostrar);
          salida.close();
        }
      }
      catch (Exception exc) {
        JOptionPane.showMessageDialog(null,
                                      "Se han producido errores al escribir en "
                                      + "el archivo " + archivo,
                                      "Error al escribir en archivo",
                                      JOptionPane.ERROR_MESSAGE);

      }
    }

    private void copiarAPortapapeles() {
      Clipboard portapapeles = getToolkit().getSystemClipboard();
      String cadenaaMostrar = pasarResultadosaCadena();
      StringSelection copiado = new StringSelection(cadenaaMostrar);
      portapapeles.setContents(copiado, copiado);
    }

  }

  class AccionCrearGrafico
      extends AbstractAction {
    public AccionCrearGrafico() {
      super();
    }

    public void actionPerformed(ActionEvent e) {
      JFreeChart chart;
      if (variarParametros) {
        XYSeriesCollection datosGrafica = new XYSeriesCollection();
        for (int c = 1; c < numCols; c++) {
          XYSeries serie = new XYSeries(nombresColumna[c]);
          for (int f = 0; f < numFilas; f++) {
            String x = (String) (datos[f][0]);
            String y = (String) (datos[f][c]);
            serie.add(Double.valueOf(x), Double.valueOf(y));
          }
          datosGrafica.addSeries(serie);
        }
        chart = ChartFactory.createXYLineChart("",
                                               nombresColumna[0],
                                               "", datosGrafica,
                                               PlotOrientation.VERTICAL,
                                               true, true, true);
        XYPlot plot = chart.getXYPlot();
        XYItemRenderer renderer = plot.getRenderer();
        BasicStroke lineaGruesa = new BasicStroke(2.0f);
        renderer.setStroke(lineaGruesa);
      } else {
        // Para cuando no hay variacin de parmetros
        DefaultCategoryDataset datosGrafica = new DefaultCategoryDataset();
        for (int c = 1; c < numCols; c++) {
          for (int f = 0; f < numFilas; f++) {
            String x = (String) (datos[f][0]);
            String y = (String) (datos[f][c]);
            datosGrafica.addValue(Double.valueOf(y), nombresColumna[c], x);
          }
        }
        chart = ChartFactory.createBarChart("",
                                            nombresColumna[0],
                                            "", datosGrafica,
                                            PlotOrientation.VERTICAL,
                                            true, true, true);
      }
      ChartFrame frame = new ChartFrame("Grfico", chart);
      frame.setIconImage(ventanaPrincipal.getIconImage());
      frame.pack();
      frame.show();
    }
  }
}

class FiltroTexto
    extends javax.swing.filechooser.FileFilter {

  public boolean accept(File f) {
    if (f.isDirectory()) {
      return true;
    }
    String extension = FramePrincipal.obtenerExtension(f);
    if (extension != null) {
      if (extension.equals("txt")) {
        return true;
      }
      else {
        return false;
      }
    }
    return false;
  }

  // The description of this filter
  public String getDescription() {
    return "Slo archivos de texto (*.txt)";
  }
}
