package interfaz;

import javax.swing.UIManager;
import java.awt.Color;
import java.io.*;
import javax.swing.LookAndFeel;

/**
 * <p>Title: VariablesPreferencias.java</p>
 * <p>Description: Las instancias de sta clase guardan las variables que
 * definen las preferencias de la aplicacin
 * @author Diego Daz Garca
 * @version 1.0
 */

public class VariablesPreferencias
    implements Serializable {
  
  static final long serialVersionUID = 7242170573454088969L;
  private String lookActual;
  private int anchoLineas;
  private double multiplicadorTamanioBloques;
  private int tamanioFuente;
  private int anchoBloque = 84; // ancho de un bloque
  private int altoBloque = 32; // alto de un bloque
  private int anchoTerminal = 8; // ancho y alto de los terminales
  private int entreBloquesY = 8; // espacio entre bloques de la misma columna (en paralelo)
  private int entreBloquesX = 16; // espacio entre bloques de la misma fila (en serie)
  private int entreFilas = 32; // espacio entre filas de bloques (serie)
// espacio entre los dos bloques que forman la figura de bloque extensible
  private int espacioExtensibles = 2;
// Espacio libre entre el dibujo y el fin del Pane, para evitar que parte del
// diagrama quede tapada por las barras de scroll
  private int margenScroll = 20;
// Es el color del fondo del diagrama
  Color colorFondo;
// Color de las lineas del diagrama
  Color colorLineas;
// Color del interior de los Bloques
  Color colorBloques;
// Color del texto
  Color colorFuente;
// Numero de decimales en pantalla y en el resultado
  private int numeroDecimalesPantalla;
  private int numeroDecimalesResultado;
// Almacenan los valores que se deben mostrar por pantalla
  private int valorMostrarFiabilidad;
  private int valorMostrarReparabilidad;

  public VariablesPreferencias() {
  }

  /**ahora se llama en Frame.jbinit, luego solo se llamar en el constructor
   * y solo se construira una vez, el resto se restaurar de disco*/
  public void inicializarValores() {
    lookActual = UIManager.getSystemLookAndFeelClassName();
    anchoLineas = 1;
    multiplicadorTamanioBloques = 1.5;
    tamanioFuente = 10;
    colorFondo = Color.white;
    colorLineas = Color.black;
    colorBloques = Color.white;
    colorFuente = Color.black;
    numeroDecimalesPantalla = 8;
    numeroDecimalesResultado = 8;
    valorMostrarFiabilidad = 0;
    valorMostrarReparabilidad = 0;
  };

  public void asignarLook(String looks) {
    lookActual = looks;
  }

  public void asignarAnchoLineas(int anchoLineas) {
    this.anchoLineas = anchoLineas;
  }

  public void asignarMultiplicadorBloques(double multiplicador) {
    multiplicadorTamanioBloques = multiplicador;
  }

  public void asignarColorFondo(Color color) {
    colorFondo = color;
  }

  public void asignarColorLineas(Color color) {
    colorLineas = color;
  }

  public void asignarColorBloques(Color color) {
    colorBloques = color;
  }

  public void asignarColorFuente(Color color) {
    colorFuente = color;
  }

  public void asignarTamanioFuente(int tamanio) {
    tamanioFuente = tamanio;
  }

  public void asignarNumeroDecimalesPantalla(int num) {
    numeroDecimalesPantalla = num;
  }

  public void asignarNumeroDecimalesResultado(int num) {
    numeroDecimalesResultado = num;
  }

  public void asignarValorMostrarFiabilidad(int num) {
    valorMostrarFiabilidad = num;
  }

  public void asignarValorMostrarReparabilidad(int num) {
    valorMostrarReparabilidad = num;
  }

  public int obtenerTamanioFuente() {
    return tamanioFuente;
  }

  public int obtenerAnchoBloques() {
    Double entero = new Double(multiplicadorTamanioBloques * anchoBloque);
    // El "+10" es para que entren decimales suficientes. Debera cambiarse
    // el anchoBloque, pero eso hace que no se puedan leer ficheros generados
    // con versiones viejas
    return entero.intValue() + 11;
  }

  public int obtenerAltoBloques() {
    Double entero = new Double(multiplicadorTamanioBloques * altoBloque);
    return entero.intValue();
  }

  public int obtenerAnchoTerminal() {
    Double entero = new Double(multiplicadorTamanioBloques * anchoTerminal);
    return entero.intValue();
  }

  public int obtenerEntreBloquesX() {
    Double entero = new Double(multiplicadorTamanioBloques * entreBloquesX);
    return entero.intValue();
  }

  public int obtenerEntreBloquesY() {
    Double entero = new Double(multiplicadorTamanioBloques * entreBloquesY);
    return entero.intValue();
  }

  public int obtenerEntreFilas() {
    Double entero = new Double(multiplicadorTamanioBloques * entreFilas);
    return entero.intValue();
  }

  public int obtenerEspacioExtensibles() {
    Double entero = new Double(multiplicadorTamanioBloques * espacioExtensibles);
    return entero.intValue();
  }

  public int obtenerMargenScroll() {
    Double entero = new Double(multiplicadorTamanioBloques * margenScroll);
    return entero.intValue();
  }

  public double obtenerMultiplicadorBloques() {
    return multiplicadorTamanioBloques;
  }

  public String obtenerLook() {
    return lookActual;
  }

  public int obtenerAnchoLineas() {
    return anchoLineas;
  }

  public Color obtenerColorFondo() {
    return colorFondo;
  }

  public Color obtenerColorLineas() {
    return colorLineas;
  }

  public Color obtenerColorBloques() {
    return colorBloques;
  }

  public Color obtenerColorFuente() {
    return colorFuente;
  }

  public int obtenerNumeroDecimalesPantalla() {
    return numeroDecimalesPantalla;
  }

  public int obtenerNumeroDecimalesResultado() {
    return numeroDecimalesResultado;
  }

  public int obtenerValorMostrarFiabilidad() {
    return valorMostrarFiabilidad;
  }

  public int obtenerValorMostrarReparabilidad() {
    return valorMostrarReparabilidad;
  }
}
