package interfaz;

import interfaz.deshacer.ElementoBloqueExpansibleEdit;
import interfaz.deshacer.ElementoBloqueSimpleEdit;
import interfaz.deshacer.ElementoEliminarEdit;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.UIManager;
import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultTreeCellEditor;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeCellEditor;
import javax.swing.tree.TreeSelectionModel;
import javax.swing.tree.TreePath;
import javax.swing.undo.UndoableEdit;

import bloques.Bloque;
import java.awt.Font;
import javax.swing.tree.TreeModel;
import java.util.Enumeration;

/**
 * <p>Title: arbolBloques.java</p>
 * <p>Description: Clase cuyo objetos son los rboles de bloques que representan
 * el diagrama</p>
 * @author Diego Daz Garca
 * @version 1.0
 */

public class arbolBloques
    extends JPanel {
  private nodo nodoRaiz;
  private nodo nodoSeleccionado = nodoRaiz;
  private Diagrama diagramaVisible;
  private FramePrincipal ventanaPrincipal;
  DefaultTreeModel modeloArbol;
  JTree arbol;
  private JScrollPane scrollPane;
  private JPopupMenu popNodo = new JPopupMenu();
  JMenuItem anadirBloque;
  JMenuItem anadirBloqueE;
  JMenuItem definirBloque;
  JMenuItem copiar;
  JMenuItem cortar;
  JMenuItem pegar;
  JMenuItem pegadoEspecial;
  JMenuItem eliminarBloque;
  JMenuItem cambiarNombre;
  EscuchadorMenusPopup mpl = new EscuchadorMenusPopup();
  MyTreeModelListener tml = new MyTreeModelListener();
  MyTreeSelectionListener tsl = new MyTreeSelectionListener();

  /**Constructor de la clase arbolBloques
   * @param bloqueRaiz Es el bloque principal del diagrama
   * @param diagramaVisible Diagrama que se est mostrando en la ventana
   * principal
   * @param ventanaPrincipal Ventana principal de la aplicacin*/
  public arbolBloques(Bloque bloqueRaiz, Diagrama diagramaVisible, FramePrincipal
                      ventanaPrincipal) {
    nodoRaiz = new nodo(bloqueRaiz, true, this);
    this.diagramaVisible = diagramaVisible;
    this.ventanaPrincipal = ventanaPrincipal;
    modeloArbol = new DefaultTreeModel(nodoRaiz);
    arbol = new JTree(modeloArbol);

    arbol.setEditable(true);
    arbol.getSelectionModel().setSelectionMode
        (TreeSelectionModel.SINGLE_TREE_SELECTION);
    arbol.setShowsRootHandles(true);

    RendererArbol rendererArbol = new RendererArbol(ventanaPrincipal);
    arbol.setCellRenderer(rendererArbol);
    TreeCellEditor t = new EditorArbol(arbol, rendererArbol);
    arbol.setCellEditor(t);
    add(arbol);

//    arbol.setBackground(UIManager.getColor("window"));
    inicializa();
  }

  /**Observador de la ventana principal
   * @return La ventana principal en la que se dibuja el rbol*/
  public FramePrincipal obtenerVentanaPrincipal() {
    return ventanaPrincipal;
  }

  /**Repinta todos los componentes del rbol*/
  public void repaint () {
    if (arbol != null) {
      arbol.setBackground(UIManager.getColor("window"));
      this.setBackground(UIManager.getColor("window"));
    }

    super.repaint();
  }

  /**Funcin observadora que devuelve el arbol de Arbolbloques
   * @return Devuelve el arbol de Arbolbloques*/
  JTree obtenerArbol(){return this.arbol;}

  /**Inicializa el arbolBloques*/
  public void inicializa() {
    modeloArbol.addTreeModelListener(tml);
    arbol.addMouseListener(mpl);
    arbol.addTreeSelectionListener(tsl);
    inicializaPopup();
  }

  /**Abre el nodo que se llame igual a la cadena que se le pase */
  public void expande(String cad) {
    expande(cad, (nodo) arbol.getModel().getRoot());
  }

  /**Recorre a partir del nodo que se le pase expandiendo el rbol hasta
   * alguno de sus hijos si coincide con la cadena que tambin se le pasa.
   */
  private void expande(String cad, nodo nodoBase) {
    Enumeration nodos = nodoBase.children();
    while (nodos.hasMoreElements()) {
      nodo nodoActual = (nodo) nodos.nextElement();
      if (nodoActual.modelo.nombre() == cad) {
        arbol.expandPath(new TreePath(nodoActual.getPath()));
      } else {
        // Slo seguimos el recorrido si no lo hemos encontrado
        expande(cad, nodoActual);
      }
    }
  }

  /**Inicializa los mens pop-up que salen al pulsar el botn derecho del ratn
   * sobre el diagrama*/
  public void inicializaPopup() {
    ActionListener eliminarBloqueListener = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        eliminaBloque();
      }
    };

    ActionListener bloqueSimpleListener = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        bloqueSimple();
      }
    };

    ActionListener bloqueExpansibleListener = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        bloqueExpansible();
      }
    };

    ActionListener cortarBloqueListener = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        cortarBloque();
      }
    };

    ActionListener copiarBloqueListener = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        copiarBloque();
      }
    };

    ActionListener pegarBloqueListener = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        pegarBloque();
      }
    };

    ActionListener pegadoEspecialBloqueListener = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        pegadoEspecial();
      }
    };

    ActionListener definirBloqueListener = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        definirBloque();
      }
    };

    ActionListener cambiarNombreListener = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        cambiarNombre();
      }
    };

    anadirBloque = new JMenuItem("Aadir bloque simple");
    anadirBloque.addActionListener(bloqueSimpleListener);
    popNodo.add(anadirBloque);
    anadirBloqueE = new JMenuItem("Aadir bloque expansible");
    popNodo.add(anadirBloqueE);
    eliminarBloque = new JMenuItem("Eliminar Bloque");
    eliminarBloque.addActionListener(eliminarBloqueListener);
    popNodo.add(eliminarBloque);
    cambiarNombre = new JMenuItem("Cambiar nombre");
    cambiarNombre.addActionListener(cambiarNombreListener);
    popNodo.add(cambiarNombre);
    popNodo.addSeparator();
    definirBloque = new JMenuItem("Definir bloque");
    definirBloque.addActionListener(definirBloqueListener);
    popNodo.add(definirBloque);
    popNodo.addSeparator();
    cortar = new JMenuItem("Cortar");
    cortar.addActionListener(cortarBloqueListener);
    popNodo.add(cortar);
    copiar = new JMenuItem("Copiar");
    copiar.addActionListener(copiarBloqueListener);
    popNodo.add(copiar);
    pegar = new JMenuItem("Pegar");
    pegar.addActionListener(pegarBloqueListener);
    popNodo.add(pegar);
    pegadoEspecial = new JMenuItem("Pegado especial");
    pegadoEspecial.addActionListener(pegadoEspecialBloqueListener);
    popNodo.add(pegadoEspecial);
    actualizaPopup();
  }

  /**Elimina del diagrama el bloque seleccionado en arbolBloques*/
  private void eliminaBloque() {
    Bloque bloquePadre = ( (nodo) (nodoSeleccionado.getParent())).modelo;
    int indiceBloque = bloquePadre.componentes.indexOf(nodoSeleccionado.modelo);
    UndoableEdit edit = new ElementoEliminarEdit(bloquePadre, indiceBloque,
                                          diagramaVisible,
                                          ventanaPrincipal.opcionesResolucion);
    bloquePadre.eliminaBloque(indiceBloque);
    interfaz.FramePrincipal.undoSupport_.postEdit(edit);
    diagramaVisible.cambiarBloqueMostrado(bloquePadre);
  }

  /**Crea un bloque simple hijo del bloque expansible seleccionado en el rbol*/
  private void bloqueSimple() {
    Bloque bloque = ( (nodo) (nodoSeleccionado)).modelo;
    UndoableEdit edit = new ElementoBloqueSimpleEdit(bloque, diagramaVisible);
    bloque.bloqueSimple();
    ventanaPrincipal.undoSupport_.postEdit(edit);
    diagramaVisible.cambiarBloqueMostrado(bloque);
  }

  /**Crea un bloque expansible hijo del bloque expansible seleccionado en el
   * rbol*/
  private void bloqueExpansible() {
    Bloque bloque = ( (nodo) (nodoSeleccionado)).modelo;
    UndoableEdit edit = new ElementoBloqueExpansibleEdit
        (bloque, diagramaVisible);
    bloque.bloqueSimple();
    ventanaPrincipal.undoSupport_.postEdit(edit);
    diagramaVisible.cambiarBloqueMostrado(bloque);
  }

  /**Corta el bloque seleccionado en el rbol*/
  private void cortarBloque() {
    Bloque bloque = ((nodo) (nodoSeleccionado)).modelo;
    diagramaVisible.cortarBloque(bloque);
  }

  /**Copia el bloque seleccionado en el rbol*/
  private void copiarBloque() {
    Bloque bloque = ((nodo) (nodoSeleccionado)).modelo;
    diagramaVisible.copiarBloque(bloque);
  }

  /**Abre el dilogo definir bloque para el bloque seleccionado en el rbol*/
  private void definirBloque() {
    Bloque bloque = ( (nodo) (nodoSeleccionado)).modelo;
    diagramaVisible.definirBloque(bloque);
  }

  /**Pega un bloque copiado o cortado anteriormente en el bloque expansible
   * seleccionado en el rbol*/
  private void pegarBloque() {
    Bloque destino = ((nodo) (nodoSeleccionado)).modelo;
    nodo nodoPadre = (nodo) nodoSeleccionado.getParent();
    Bloque padreDestino;
    if (nodoPadre != null)
      padreDestino = nodoPadre.modelo;
    else
      padreDestino = null;
    diagramaVisible.pegarBloque(destino, padreDestino, Diagrama.PEGADO);
  }

  /**Realiza el pegado especial de un bloque copiado anteriormente en el bloque
   * expansible seleccionado en el rbol*/
  private void pegadoEspecial() {
    Bloque destino = ((nodo) (nodoSeleccionado)).modelo;
    nodo nodoPadre = (nodo) nodoSeleccionado.getParent();
    Bloque padreDestino;
    if (nodoPadre != null)
      padreDestino = nodoPadre.modelo;
    else
      padreDestino = null;
    diagramaVisible.pegadoEspecial(destino, padreDestino);
  }

  private void cambiarNombre() {
    TreePath nodoActivo = arbol.getSelectionModel().getSelectionPath();
    if (nodoActivo != null)
      this.arbol.startEditingAtPath(nodoActivo);
  }

  /**Actualiza el men pop-up del rbol*/
  public void actualizaPopup() {
    Bloque bloque = null;
    if (nodoSeleccionado != null) {
      bloque = ( (nodo) (nodoSeleccionado)).modelo;
    }
    eliminarBloque.setEnabled( (bloque != null) &&
                              ! (nodoRaiz.getUserObject() == bloque));
    cambiarNombre.setEnabled((bloque != null));
    anadirBloque.setEnabled( (bloque != null) && (bloque.esExtensible()));
    anadirBloqueE.setEnabled( (bloque != null) && (bloque.esExtensible()));
    definirBloque.setEnabled( (bloque != null) && (!bloque.esExtensible()));
    // El bloque.padre() != null siguiente es para evitar que se elmine al
    // cortar el bloque raz
    cortar.setEnabled(bloque != null && bloque.padre() != null);
    copiar.setEnabled(bloque != null);
    pegar.setEnabled(diagramaVisible.puedePegar());
    pegadoEspecial.setEnabled(diagramaVisible.puedePegar());
  }

  class MyTreeModelListener
      implements TreeModelListener {
    public void treeNodesChanged(TreeModelEvent e) {
      nodo nodoCambiado;
      nodoCambiado = (nodo) (e.getTreePath().getLastPathComponent());
      try {
        int index = e.getChildIndices()[0];
        nodoCambiado = (nodo) (nodoCambiado.getChildAt(index));
      }
      catch (NullPointerException exc) {}
      nodoCambiado.actualizaNombre();
    }

    public void treeNodesInserted(TreeModelEvent e) {}

    public void treeNodesRemoved(TreeModelEvent e) {}

    public void treeStructureChanged(TreeModelEvent e) {}
  }

  class MyTreeSelectionListener
      implements TreeSelectionListener {
    public void valueChanged(TreeSelectionEvent e) {
      nodoSeleccionado = (nodo) arbol.getLastSelectedPathComponent();
    }
  }

  void mostrarMenuPopup(MouseEvent e) {
    popNodo.setVisible(true);
    popNodo.show( (JTree) e.getSource(), e.getX(), e.getY());
  }

  class EscuchadorMenusPopup
      extends MouseAdapter {
    public void MousePressed(MouseEvent e) {}

    public void mouseReleased(MouseEvent e) {
      if (e.isPopupTrigger()) {
        actualizaPopup();
        mostrarMenuPopup(e);
      }
      else if (e.getClickCount() >= 2) { // Doble click o mas
        if (nodoSeleccionado != null) {
          if (nodoSeleccionado.modelo.esExtensible()) {
            diagramaVisible.cambiarBloqueMostrado(nodoSeleccionado.modelo);
          }
          else {
            diagramaVisible.definirBloque(nodoSeleccionado.modelo);
          }
        }
      }
    }
  }
}

class RendererArbol
    extends DefaultTreeCellRenderer {
  Icon serie, paralelo, basico, mostradoSerie, mostradoParalelo;
  FramePrincipal ventanaPrincipal;

  public RendererArbol(FramePrincipal ventanaPrincipal) {
    this.ventanaPrincipal = ventanaPrincipal;

    try {
      serie = new ImageIcon(interfaz.FramePrincipal.class.getResource(
          "iconos/arbol_serie.gif"));
      paralelo = new ImageIcon(interfaz.FramePrincipal.class.getResource(
          "iconos/arbol_paralelo.gif"));
      basico = new ImageIcon(interfaz.FramePrincipal.class.getResource(
          "iconos/arbol_basico.gif"));
      mostradoSerie = new ImageIcon(interfaz.FramePrincipal.class.getResource(
          "iconos/serie_nuevo.gif"));
      mostradoParalelo = new ImageIcon(interfaz.FramePrincipal.class.getResource(
          "iconos/paralelo_nuevo.gif"));
    } catch (Exception e) {
      JOptionPane.showMessageDialog(null,
                                     "No se han podido cargar los iconos del "
                                     +"rbol",
                                     "Error al inicializar el rbol",
                                     JOptionPane.ERROR_MESSAGE);
    }
  }

  public Component getTreeCellRendererComponent(JTree arbol, Object valor,
                                                boolean seleccionado,
                                                boolean expandido,
                                                boolean hoja, int fila,
                                                boolean tieneFoco) {
    super.getTreeCellRendererComponent(arbol, valor, seleccionado, expandido,
                                       hoja,
                                       fila, tieneFoco);
    nodo nodoActual = (nodo) valor;
    Bloque bloque = (Bloque) nodoActual.modelo;
    Font fuente = getFont();
    Font fuenteNegrita = fuente.deriveFont(Font.BOLD);
    Font fuenteNormal = fuente.deriveFont(Font.PLAIN);

    if ((ventanaPrincipal != null)
        && (nodoActual.modelo.nombre() ==
            ventanaPrincipal.obtenerBloqueRepresentado().nombre())) {
      if (nodoActual.modelo.esParalelo()) {
        setIcon(mostradoParalelo);
        setFont(fuenteNegrita);
      } else {
        setIcon(mostradoSerie);
        setFont(fuenteNegrita);
      }
      return this;
    }
    if (bloque.esExtensible()) {
      if (bloque.esParalelo()) {
        setIcon(paralelo);
        setFont(fuenteNormal);
      }
      else {
        setIcon(serie);
        setFont(fuenteNormal);
      }
    }
    else {
      setIcon(basico);
      setFont(fuenteNormal);
    }
    return this;
  }
}

class EditorArbol
    extends DefaultTreeCellEditor {

  public EditorArbol(JTree arbol, DefaultTreeCellRenderer rendererArbol) {
    super(arbol, rendererArbol);
  }

  protected void determineOffset(JTree arbol, Object valor,
                                 boolean seleccionado,
                                 boolean expandido,
                                 boolean hoja, int fila) {
    JLabel l = (JLabel) renderer.getTreeCellRendererComponent(arbol, valor,
        seleccionado, expandido, hoja, fila, arbol.hasFocus());
    editingIcon = l.getIcon();
    offset = renderer.getIconTextGap() + editingIcon.getIconWidth();
  }
}
