package interfaz.deshacer;

import javax.swing.undo.*;
import bloques.BloqueBasico;
import util.DatosBloque;

/**
 * <p>Title: ElementoDefinirBloqueEdit</p>
 * <p>Description: Clase para la funcionalidad deshacer-rehacer de Definir
 * bloque</p>
 * @author Diego Daz Garca
 * @version 1.0
 */

public class ElementoDefinirBloqueEdit
    extends AbstractUndoableEdit {
  BloqueBasico bloqueRedefinido;
  DatosBloque datosAnteriores;
  DatosBloque datosFinales;

  public ElementoDefinirBloqueEdit(BloqueBasico bloque,
                                   DatosBloque datosAnteriores,
                                   DatosBloque datosFinales) {
    bloqueRedefinido = bloque;
    this.datosAnteriores = datosAnteriores;
    this.datosFinales = datosFinales;
  }

  public void undo() throws CannotUndoException {

    if (datosAnteriores.definidoLambda()) {
      bloqueRedefinido.asignarLambda(datosAnteriores.metricaFallos());
    }
    else {
      bloqueRedefinido.asignarMttf(datosAnteriores.metricaFallos());
    }

    if (datosAnteriores.definidoMu()) {
      bloqueRedefinido.asignarMu(datosAnteriores.metricaReparabilidad());
    }
    else {
      bloqueRedefinido.asignarMttr(datosAnteriores.metricaReparabilidad());

    }

    bloqueRedefinido.asignarMtbf(datosAnteriores.valorMtbf());
  }

  public void redo() throws CannotUndoException {

    if (datosFinales.definidoLambda()) {
      bloqueRedefinido.asignarLambda(datosFinales.metricaFallos());
    }
    else {
      bloqueRedefinido.asignarMttf(datosFinales.metricaFallos());
    }

    if (datosFinales.definidoMu()) {
      bloqueRedefinido.asignarMu(datosFinales.metricaReparabilidad());
    }
    else {
      bloqueRedefinido.asignarMttr(datosFinales.metricaReparabilidad());
    }

    bloqueRedefinido.asignarMtbf(datosFinales.valorMtbf());
  }

  public boolean canUndo() {
    return true;
  }

  public boolean canRedo() {
    return true;
  }

  public String getPresentationName() {
    return "Definir bloque";
  }
}
