package util;

import bloques.BloqueBasico;

/**
 * <p>Title: DatosBloque.java</p>
 * <p>Description: Clase cuyas instancias guardarn los datos necesarios para
 * volver al estado inicial un bloque que ha sido modificado a travs de la
 * pantalla definir bloque </p>
 * @author Diego Daz Garca
 * @version 1.0
 */

public class DatosBloque {
  private boolean reparable;
  private boolean definidoLambda; // Si no se asigna lambda, se ha asignado MTTF
  private boolean definidoMu; // Si no se asigna Mu, se ha asignado MTTR
  private double metricaFallos; // Lambda o MTTF
  private double metricaReparabilidad; // Mu o MTTR
  private double Mtbf;

  /**Constructor de la clase
   * @param bloque Bloque del cual se guardarn los datos
   * @param reparable Indica si el sistema en el que est es reparable*/
  public DatosBloque(BloqueBasico bloque, boolean reparable) {
    this.reparable = reparable;
    if (bloque.definidoMTTF()) {
      definidoLambda = false;
      metricaFallos = bloque.obtenerMttf(reparable);
    }
    else {
      definidoLambda = true;
      metricaFallos = bloque.obtenerLambda(reparable);
    }
    if (bloque.definidoMTTR()) {
      definidoMu = false;
      metricaReparabilidad = bloque.obtenerMttr();
    }
    else {
      definidoMu = true;
      metricaReparabilidad = bloque.obtenerMu();
    }
    Mtbf = bloque.obtenerMtbf(reparable);
  }

  /**Indica qu dato se haba definido para la fiabilidad del bloque
   * @return Cierto si se defini la tasa de fallos, falso si fue el mttf*/
  public boolean definidoLambda() {
    return definidoLambda;
  }

  /**Indica qu dato se haba definido para la reparabilidad del bloque
   * @return Cierto si se defini la tasa de reparacin, falso si fue el
   * mttr*/
  public boolean definidoMu() {
    return definidoMu;
  }

  /**Devuelve el valor asignado para la fiabilidad
   * @return El valor de lambda o mttf segn sea el caso*/
  public double metricaFallos() {
    return metricaFallos;
  }

  /**Devuelve el valor asignado para la reparabilidad
   * @return El valor de mu o mttr segn sea el caso*/
  public double metricaReparabilidad() {
    return metricaReparabilidad;
  }

  /**Devuelve el valor asignado para el mtbf
   * @return El valor de mtbf*/
  public double valorMtbf() {
    return Mtbf;
  }
}